@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2001-2005 by Robby Stephenson
+    copyright            : (C) 2001-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -15,9 +15,13 @@
 #include "field.h"
 #include "entry.h"
 #include "tellico_debug.h"
+#include "latin1literal.h"
+#include "tellico_utils.h"
+#include "controller.h"
+#include "collectionfactory.h"
+#include "stringset.h"
 
 #include <klocale.h>
-#include <kglobal.h> // for KMAX
 
 #include <qregexp.h>
 
@@ -26,17 +30,20 @@ using Tellico::Data::Collection;
 const QString Collection::s_emptyGroupTitle = i18n("(Empty)");
 const QString Collection::s_peopleGroupName = QString::fromLatin1("_people");
 
-Collection::Collection(const QString& title_, const QString& entryName_, const QString& entryTitle_)
-    : QObject(), KShared(), m_title(title_), m_entryName(entryName_), m_entryTitle(entryTitle_), m_entryIdDict(101) {
+Collection::Collection(const QString& title_, const QString& entryTitle_)
+    : QObject(), KShared(), m_nextEntryId(0), m_title(title_), m_entryTitle(entryTitle_), m_entryIdDict(997) {
   m_entryGroupDicts.setAutoDelete(true);
 
   m_id = getID();
-  m_iconName = entryName_ + 's';
 }
 
 Collection::~Collection() {
 }
 
+QString Collection::typeName() const {
+  return CollectionFactory::typeName(type());
+}
+
 bool Collection::addFields(FieldVec list_) {
   bool success = true;
   for(FieldVec::Iterator it = list_.begin(); it != list_.end(); ++it) {
@@ -45,26 +52,27 @@ bool Collection::addFields(FieldVec list_) {
   return success;
 }
 
-bool Collection::addField(Field* field_) {
+bool Collection::addField(FieldPtr field_) {
   if(!field_) {
     return false;
   }
 
-  // fieldByName() returns 0 if there's no field by that name
   // this essentially checks for duplicates
-  if(fieldByName(field_->name())) {
+  if(hasField(field_->name())) {
     myDebug() << "Collection::addField() - replacing " << field_->name() << endl;
     removeField(fieldByName(field_->name()), true);
   }
 
-//  kdDebug() << "Collection::addField() - adding " << field_->name() << endl;
+//  myDebug() << "Collection::addField() - adding " << field_->name() << endl;
   m_fields.append(field_);
   if(field_->formatFlag() == Field::FormatName) {
     m_peopleFields.append(field_); // list of people attributes
     if(m_peopleFields.count() > 1) {
       // the second time that a person field is added, add a "pseudo-group" for people
       if(m_entryGroupDicts.find(s_peopleGroupName) == 0) {
-        m_entryGroupDicts.insert(s_peopleGroupName, new EntryGroupDict());
+        EntryGroupDict* d = new EntryGroupDict();
+        d->setAutoDelete(true);
+        m_entryGroupDicts.insert(s_peopleGroupName, d);
         m_entryGroups.prepend(s_peopleGroupName);
       }
     }
@@ -84,7 +92,7 @@ bool Collection::addField(Field* field_) {
   if(field_->flags() & Field::AllowGrouped) {
     // m_entryGroups autoDeletes each QDict when the Collection d'tor is called
     EntryGroupDict* dict = new EntryGroupDict();
-    // don't autoDelete, since the group is deleted when it becomes empty
+    dict->setAutoDelete(true);
     m_entryGroupDicts.insert(field_->name(), dict);
     // cache the possible groups of entries
     m_entryGroups << field_->name();
@@ -101,29 +109,29 @@ bool Collection::addField(Field* field_) {
   return true;
 }
 
-bool Collection::mergeField(Field* newField_) {
+bool Collection::mergeField(FieldPtr newField_) {
   if(!newField_) {
     return false;
   }
 
-  Field* currField = fieldByName(newField_->name());
+  FieldPtr currField = fieldByName(newField_->name());
   if(!currField) {
     // does not exist in current collection, add it
-    return addField(newField_->clone());
+    Data::FieldPtr f = new Field(*newField_);
+    bool success = addField(f);
+    Controller::self()->addedField(this, f);
+    return success;
+  }
+
+  if(newField_->type() == Field::Table2) {
+    newField_->setType(Data::Field::Table);
+    newField_->setProperty(QString::fromLatin1("columns"), QChar('2'));
   }
 
   // the original field type is kept
   if(currField->type() != newField_->type()) {
-    // make special exception for music collections and the track field, ok
-    // to convert Table to Table2
-    if(type() == Album && currField->type() == Field::Table && newField_->type() == Field::Table2) {
-//      currField->setType(Field::Table2);
-      newField_->setType(Data::Field::Table);
-      currField->setProperty(QString::fromLatin1("columns"), QChar('2'));
-    } else {
-      kdDebug() << "Collection::mergeField() - skipping, field type mismatch for " << currField->title() << endl;
-      return false;
-    }
+    myDebug() << "Collection::mergeField() - skipping, field type mismatch for " << currField->title() << endl;
+    return false;
   }
 
   // if field is a Choice, then make sure all values are there
@@ -147,8 +155,31 @@ bool Collection::mergeField(Field* newField_) {
 
   // if new field has additional extended properties, add those
   for(StringMap::ConstIterator it = newField_->propertyList().begin(); it != newField_->propertyList().end(); ++it) {
-    if(currField->property(it.key()).isEmpty()) {
-      currField->setProperty(it.key(), it.data());
+    const QString propName = it.key();
+    const QString currValue = currField->property(propName);
+    if(currValue.isEmpty()) {
+      currField->setProperty(propName, it.data());
+    } else if (it.data() != currValue) {
+      if(currField->type() == Field::URL && propName == Latin1Literal("relative")) {
+        kdWarning() << "Collection::mergeField() - relative URL property does not match for " << currField->name() << endl;
+      } else if((currField->type() == Field::Table && propName == Latin1Literal("columns"))
+             || (currField->type() == Field::Rating && propName == Latin1Literal("maximum"))) {
+        bool ok;
+        uint currNum = Tellico::toUInt(currValue, &ok);
+        uint newNum = Tellico::toUInt(it.data(), &ok);
+        if(newNum > currNum) { // bigger values
+          myDebug() << "Collection::mergeField() - changing " << propName << " for " << currField->name() << endl;
+          currField->setProperty(propName, QString::number(newNum));
+        }
+      } else if(currField->type() == Field::Rating && propName == Latin1Literal("minimum")) {
+        bool ok;
+        uint currNum = Tellico::toUInt(currValue, &ok);
+        uint newNum = Tellico::toUInt(it.data(), &ok);
+        if(newNum < currNum) { // smaller values
+          myDebug() << "Collection::mergeField() - changing " << propName << " for " << currField->name() << endl;
+          currField->setProperty(propName, QString::number(newNum));
+        }
+      }
     }
   }
 
@@ -157,19 +188,19 @@ bool Collection::mergeField(Field* newField_) {
   return true;
 }
 
-// be really careful with these field pointers, try not to call to many other functions
+// be really careful with these field pointers, try not to call too many other functions
 // which may depend on the field list
-bool Collection::modifyField(Field* newField_) {
+bool Collection::modifyField(FieldPtr newField_) {
   if(!newField_) {
     return false;
   }
-//  kdDebug() << "Collection::modifyField() - " << newField_->name() << endl;
+//  myDebug() << "Collection::modifyField() - " << newField_->name() << endl;
 
 // the field name never changes
   const QString& fieldName = newField_->name();
-  Field* oldField = fieldByName(fieldName);
+  FieldPtr oldField = fieldByName(fieldName);
   if(!oldField) {
-    kdDebug() << "Collection::modifyField() - no field named " << fieldName << endl;
+    myDebug() << "Collection::modifyField() - no field named " << fieldName << endl;
     return false;
   }
 
@@ -194,7 +225,7 @@ bool Collection::modifyField(Field* newField_) {
     m_fields.insert(it, newField_);
     m_fields.remove(oldField);
   } else {
-    kdDebug() << "Collection::modifyField() - no index found!" << endl;
+    myDebug() << "Collection::modifyField() - no index found!" << endl;
     return false;
   }
 
@@ -234,7 +265,9 @@ bool Collection::modifyField(Field* newField_) {
   if(isPeople) {
     // if there's more than one people field and no people dict exists yet, add it
     if(m_peopleFields.count() > 1 && !m_entryGroupDicts.find(s_peopleGroupName)) {
-      m_entryGroupDicts.insert(s_peopleGroupName, new EntryGroupDict());
+      EntryGroupDict* d = new EntryGroupDict();
+      d->setAutoDelete(true);
+      m_entryGroupDicts.insert(s_peopleGroupName, d);
       // put it at the top of the list
       m_entryGroups.prepend(s_peopleGroupName);
     }
@@ -257,7 +290,9 @@ bool Collection::modifyField(Field* newField_) {
 //      m_entryGroupDicts.replace(fieldName, new EntryGroupDict());
     }
   } else if(isGrouped) {
-    m_entryGroupDicts.insert(fieldName, new EntryGroupDict());
+    EntryGroupDict* d = new EntryGroupDict();
+    d->setAutoDelete(true);
+    m_entryGroupDicts.insert(fieldName, d);
     // cache the possible groups of entries
     m_entryGroups << fieldName;
     resetGroups = true;
@@ -270,7 +305,10 @@ bool Collection::modifyField(Field* newField_) {
     m_imageFields.append(newField_);
   }
 
+
+
   if(resetGroups) {
+    myLog() << "Collection::modifyField() - invalidating groups" << endl;
     invalidateGroups();
   }
 
@@ -287,11 +325,11 @@ bool Collection::removeField(const QString& name_, bool force_) {
 }
 
 // force allows me to force the deleting of the title field if I need to
-bool Collection::removeField(Field* field_, bool force_/*=false*/) {
+bool Collection::removeField(FieldPtr field_, bool force_/*=false*/) {
   if(!field_ || !m_fields.contains(field_)) {
     return false;
   }
-//  kdDebug() << "Collection::deleteField() - name = " << field_->name() << endl;
+//  myDebug() << "Collection::removeField() - name = " << field_->name() << endl;
 
   // can't delete the title field
   if((field_->flags() & Field::NoDelete) && !force_) {
@@ -302,6 +340,7 @@ bool Collection::removeField(Field* field_, bool force_/*=false*/) {
   if(field_->formatFlag() == Field::FormatName) {
     success &= m_peopleFields.remove(field_);
   }
+
   if(field_->type() == Field::Image) {
     success &= m_imageFields.remove(field_);
   }
@@ -316,7 +355,7 @@ bool Collection::removeField(Field* field_, bool force_/*=false*/) {
 
   for(EntryVecIt it = m_entries.begin(); it != m_entries.end(); ++it) {
     // setting the fields to an empty string removes the value from the entry's list
-    it->setField(field_->name(), QString::null);
+    it->setField(field_, QString::null);
   }
 
   if(field_->flags() & Field::AllowGrouped) {
@@ -328,8 +367,6 @@ bool Collection::removeField(Field* field_, bool force_/*=false*/) {
   }
 
   success &= m_fields.remove(field_);
-
-//  delete field_; // removeFieldCommand will delete the field
   return success;
 }
 
@@ -346,83 +383,89 @@ void Collection::reorderFields(const FieldVec& list_) {
   }
 }
 
-void Collection::addEntry(Entry* entry_) {
+void Collection::addEntry(EntryPtr entry_) {
   if(!entry_) {
     return;
   }
 
-#ifndef NDEBUG
-  if(entry_->collection() != this) {
-    // should the addEntry() call be done in the Entry constructor?
-    kdWarning() << "Collection::addEntry() - entry is not being added to its parent collection" << endl;
+  if(this != entry_->collection()) {
+    entry_->setCollection(this);
   }
-#endif
 
   m_entries.append(entry_);
-//  kdDebug() << "Collection::addEntry() - added entry (" << entry_->title() << ")" << endl;
+//  myDebug() << "Collection::addEntry() - added entry (" << entry_->title() << ")" << endl;
 
+  if(entry_->id() >= m_nextEntryId) {
+    m_nextEntryId = entry_->id() + 1;
+  } else if(entry_->id() == -1) {
+    entry_->setId(m_nextEntryId);
+    ++m_nextEntryId;
+  } else if(m_entryIdDict.find(entry_->id())) {
+    myDebug() << "Collection::addEntry() - the collection already has an entry with id = " << entry_->id() << endl;
+    entry_->setId(m_nextEntryId);
+    ++m_nextEntryId;
+  }
+  m_entryIdDict.insert(entry_->id(), entry_);
   populateDicts(entry_);
-  int id = KMAX(1, entry_->id()); // id is -1 for entries as created, but not added
-  while(m_entryIdDict[id]) {
-    ++id;
-  }
-  m_entryIdDict.insert(id, entry_);
-  entry_->setId(id);
 }
 
-void Collection::removeEntryFromDicts(Entry* entry_) {
-  // need a copy of the vector since it gets changed
-  PtrVector<EntryGroup> groups = entry_->groups();
-  for(PtrVector<EntryGroup>::Iterator group = groups.begin(); group != groups.end(); ++group) {
-    if(entry_->removeFromGroup(group.ptr())) {
-      emit signalGroupModified(this, group.ptr());
-    }
-    if(group->isEmpty()) {
-      EntryGroupDict* dict = m_entryGroupDicts.find(group->fieldName());
-      if(!dict) {
-        continue;
+void Collection::removeEntriesFromDicts(EntryVec entries_) {
+  PtrVector<EntryGroup> modifiedGroups;
+  for(EntryVecIt entry = entries_.begin(); entry != entries_.end(); ++entry) {
+    // need a copy of the vector since it gets changed
+    PtrVector<EntryGroup> groups = entry->groups();
+    for(PtrVector<EntryGroup>::Iterator group = groups.begin(); group != groups.end(); ++group) {
+      if(entry->removeFromGroup(group.ptr()) && !modifiedGroups.contains(group.ptr())) {
+        modifiedGroups.push_back(group.ptr());
+      }
+      if(group->isEmpty() && !m_groupsToDelete.contains(group.ptr())) {
+        m_groupsToDelete.push_back(group.ptr());
       }
-      dict->remove(group->groupName());
-      delete group.ptr();
     }
   }
+  for(PtrVector<EntryGroup>::Iterator group = modifiedGroups.begin(); group != modifiedGroups.end(); ++group) {
+    emit signalGroupModified(this, group.ptr());
+  }
 }
 
-// this function gets called whenever a entry is modified. Its purpose is to keep the
+// this function gets called whenever an entry is modified. Its purpose is to keep the
 // groupDicts current. It first removes the entry from every group to which it belongs,
 // then it repopulates the dicts with the entry's fields
-void Collection::updateDicts(Entry* entry_) {
+void Collection::updateDicts(EntryVec entries_) {
 //  myDebug() << "Collection::updateDicts()" << endl;
-  if(!entry_) {
+  if(entries_.isEmpty()) {
     return;
   }
 
-  removeEntryFromDicts(entry_);
-  populateDicts(entry_);
+  removeEntriesFromDicts(entries_);
+  for(EntryVecIt entry = entries_.begin(); entry != entries_.end(); ++entry) {
+    populateDicts(entry);
+  }
+  cleanGroups();
 }
 
-bool Collection::removeEntry(Entry* entry_) {
+bool Collection::removeEntry(EntryPtr entry_) {
   if(!entry_) {
     return false;
   }
 
-//  kdDebug() << "Collection::deleteEntry() - deleted entry - " << entry_->title() << endl;
-  removeEntryFromDicts(entry_);
+//  myDebug() << "Collection::deleteEntry() - deleted entry - " << entry_->title() << endl;
+  removeEntriesFromDicts(EntryVec(entry_));
   bool success = m_entryIdDict.remove(entry_->id());
 
-  success &= m_entries.remove(entry_);;
-
+  success &= m_entries.remove(entry_);
+  cleanGroups();
   return success;
 }
 
 Tellico::Data::FieldVec Collection::fieldsByCategory(const QString& cat_) {
 #ifndef NDEBUG
   if(m_fieldCategories.findIndex(cat_) == -1) {
-    kdDebug() << "Collection::fieldsByCategory() - '" << cat_ << "' is not in category list" << endl;
+    myDebug() << "Collection::fieldsByCategory() - '" << cat_ << "' is not in category list" << endl;
   }
 #endif
   if(cat_.isEmpty()) {
-    kdDebug() << "Collection::fieldsByCategory() - empty category!" << endl;
+    myDebug() << "Collection::fieldsByCategory() - empty category!" << endl;
     return FieldVec();
   }
 
@@ -439,9 +482,8 @@ const QString& Collection::fieldNameByTitle(const QString& title_) const {
   if(title_.isEmpty()) {
     return QString::null;
   }
-  Field* f = fieldByTitle(title_);
-  if(!f) {
-    kdWarning() << "Collection::fieldNameByTitle() - no field titled " << title_ << endl;
+  FieldPtr f = fieldByTitle(title_);
+  if(!f) { // might happen in MainWindow::saveCollectionOptions
     return QString::null;
   }
   return f->name();
@@ -451,7 +493,7 @@ const QString& Collection::fieldTitleByName(const QString& name_) const {
   if(name_.isEmpty()) {
     return QString::null;
   }
-  Field* f = fieldByName(name_);
+  FieldPtr f = fieldByName(name_);
   if(!f) {
     kdWarning() << "Collection::fieldTitleByName() - no field named " << name_ << endl;
     return QString::null;
@@ -465,35 +507,30 @@ QStringList Collection::valuesByFieldName(const QString& name_) const {
   }
   bool multiple = (fieldByName(name_)->flags() & Field::AllowMultiple);
 
-  QStringList strlist;
+  StringSet values;
   for(EntryVec::ConstIterator it = m_entries.begin(); it != m_entries.end(); ++it) {
     if(multiple) {
-      strlist += it->fields(name_, false);
+      values.add(it->fields(name_, false));
     } else {
-      strlist += it->field(name_);
+      values.add(it->field(name_));
     }
   } // end entry loop
-  strlist.sort();
 
-  QStringList::Iterator it = strlist.begin();
-  while(it != strlist.end()) {
-    const QString& s = *it;
-    ++it;
-    while(it != strlist.end() && s == *it) {
-      it = strlist.remove(it);
-    }
-  }
-  return strlist;
+  return values.toList();
 }
 
-Tellico::Data::Field* const Collection::fieldByName(const QString& name_) const {
+Tellico::Data::FieldPtr Collection::fieldByName(const QString& name_) const {
   return m_fieldNameDict.isEmpty() ? 0 : m_fieldNameDict.find(name_);
 }
 
-Tellico::Data::Field* const Collection::fieldByTitle(const QString& title_) const {
+Tellico::Data::FieldPtr Collection::fieldByTitle(const QString& title_) const {
   return m_fieldTitleDict.isEmpty() ? 0 : m_fieldTitleDict.find(title_);
 }
 
+bool Collection::hasField(const QString& name_) const {
+  return !name_.isEmpty() && fieldByName(name_) != 0;
+}
+
 bool Collection::isAllowed(const QString& key_, const QString& value_) const {
   // empty string is always allowed
   if(value_.isEmpty()) {
@@ -501,7 +538,7 @@ bool Collection::isAllowed(const QString& key_, const QString& value_) const {
   }
 
   // find the field with a name of 'key_'
-  Field* field = fieldByName(key_);
+  FieldPtr field = fieldByName(key_);
 
   // if the type is not multiple choice or if value_ is allowed, return true
   if(field && (field->type() != Field::Choice || field->allowed().findIndex(value_) > -1)) {
@@ -515,7 +552,7 @@ Tellico::Data::EntryGroupDict* const Collection::entryGroupDictByName(const QStr
   return m_entryGroupDicts.isEmpty() ? 0 : m_entryGroupDicts.find(name_);
 }
 
-void Collection::populateDicts(Entry* entry_) {
+void Collection::populateDicts(EntryPtr entry_) {
   if(m_entryGroupDicts.isEmpty()) {
     return;
   }
@@ -529,73 +566,50 @@ void Collection::populateDicts(Entry* entry_) {
     EntryGroupDict* dict = dictIt.current();
     // the field name might be the people group name
     QString fieldName = dictIt.currentKey();
-    bool isBool = fieldByName(fieldName) && fieldByName(fieldName)->type() == Field::Bool;
+    bool isBool = hasField(fieldName) && fieldByName(fieldName)->type() == Field::Bool;
 
     QStringList groups = entryGroupNamesByField(entry_, fieldName);
     for(QStringList::ConstIterator groupIt = groups.begin(); groupIt != groups.end(); ++groupIt) {
       // find the group for this group name
       // bool fields used the field title
-      EntryGroup* group;
-      if(isBool && *groupIt != s_emptyGroupTitle) {
-        group = dict->find(fieldTitleByName(fieldName));
-      } else {
-        group = dict->find(*groupIt);
+      QString groupTitle = *groupIt;
+      if(isBool && groupTitle != s_emptyGroupTitle) {
+        groupTitle = fieldTitleByName(fieldName);
       }
+      EntryGroup* group = dict->find(groupTitle);
       // if the group doesn't exist, create it
       if(!group) {
-        // if it's a bool, rather than showing "true", use field title instead of "true"
-        // as long as it's not the empty group name
-        if(isBool && *groupIt != s_emptyGroupTitle) {
-          QString t = fieldTitleByName(fieldName);
-          group = new EntryGroup(t, fieldName);
-          dict->insert(t, group);
-        } else {
-          group = new EntryGroup(*groupIt, fieldName);
-          dict->insert(*groupIt, group);
-        }
+        group = new EntryGroup(groupTitle, fieldName);
+        dict->insert(groupTitle, group);
+      } else if(group->isEmpty()) {
+        // if it's empty, then it was added to the vector of groups to delete
+        // remove it from that vector now that we're adding to it
+        m_groupsToDelete.remove(group);
       }
       if(entry_->addToGroup(group)) {
         emit signalGroupModified(this, group);
       }
     } // end group loop
-//    kdDebug() << "Collection::populateDicts - end of group loop" << endl;
+//    myDebug() << "Collection::populateDicts - end of group loop" << endl;
   } // end dict loop
-//  kdDebug() << "Collection::populateDicts - end of full loop" << endl;
+//  myDebug() << "Collection::populateDicts - end of full loop" << endl;
 }
 
 // return a string list for all the groups that the entry belongs to
 // for a given field. Normally, this would just be splitting the entry's value
 // for the field, but if the field name is the people pseudo-group, then it gets
 // a bit more complicated
-QStringList Collection::entryGroupNamesByField(Entry* entry_, const QString& fieldName_) {
+QStringList Collection::entryGroupNamesByField(EntryPtr entry_, const QString& fieldName_) {
   if(fieldName_ != s_peopleGroupName) {
     return entry_->groupNamesByFieldName(fieldName_);
   }
 
-  QStringList groups;
+  StringSet values;
   for(FieldVec::Iterator it = m_peopleFields.begin(); it != m_peopleFields.end(); ++it) {
-    groups += entry_->groupNamesByFieldName(it->name());
+    values.add(entry_->groupNamesByFieldName(it->name()));
   }
-
-  // just want unique values
-  // so if there's more than one people field, need to remove any duplicates
-  if(m_peopleFields.count() > 1) {
-    groups.sort();
-    unsigned i = 1;
-    while(i < groups.count()) {
-      if(groups[i] == groups[i-1]) {
-        groups.remove(groups.at(i));
-      } else {
-        ++i;
-      }
-    }
-  }
-
-  // don't want the empty group
-  // effectively, this means that if the people group is used, entries with no
-  // people are not shown in the group view
-  groups.remove(s_emptyGroupTitle);
-  return groups;
+  values.remove(s_emptyGroupTitle);
+  return values.toList();
 }
 
 void Collection::invalidateGroups() {
@@ -604,27 +618,28 @@ void Collection::invalidateGroups() {
     dictIt.current()->clear();
   }
 
+  // populateDicts() will make signals that the group view is connected to, block those
+  blockSignals(true);
   for(EntryVecIt it = m_entries.begin(); it != m_entries.end(); ++it) {
     it->invalidateFormattedFieldValue();
-    // populateDicts() will make signals that the group view is connected to, block those
-    blockSignals(true);
+    it->clearGroups();
     populateDicts(it);
-    blockSignals(false);
   }
+  blockSignals(false);
 }
 
-Tellico::Data::Entry* Collection::entryById(int id_) {
+Tellico::Data::EntryPtr Collection::entryById(long id_) {
   return m_entryIdDict[id_];
 }
 
-void Collection::addBorrower(Data::Borrower* borrower_) {
+void Collection::addBorrower(Data::BorrowerPtr borrower_) {
   if(!borrower_) {
     return;
   }
   m_borrowers.append(borrower_);
 }
 
-void Collection::addFilter(Filter* filter_) {
+void Collection::addFilter(FilterPtr filter_) {
   if(!filter_) {
     return;
   }
@@ -632,7 +647,7 @@ void Collection::addFilter(Filter* filter_) {
   m_filters.append(filter_);
 }
 
-bool Collection::removeFilter(Filter* filter_) {
+bool Collection::removeFilter(FilterPtr filter_) {
   if(!filter_) {
     return false;
   }
@@ -642,9 +657,103 @@ bool Collection::removeFilter(Filter* filter_) {
   return true;
 }
 
+void Collection::clear() {
+  // since the collection holds a pointer to each entry and each entry
+  // hold a pointer to the collection, and they're both sharedptrs,
+  // neither will ever get deleted, unless the collection removes
+  // all held pointers, specifically to entries
+  m_fields.clear();
+  m_peopleFields.clear();
+  m_imageFields.clear();
+  m_fieldNameDict.clear();
+  m_fieldTitleDict.clear();
+  m_entries.clear();
+  m_entryIdDict.clear();
+  m_entryGroupDicts.clear();
+  m_groupsToDelete.clear();
+  m_filters.clear();
+  m_borrowers.clear();
+}
+
+void Collection::cleanGroups() {
+  for(PtrVector<EntryGroup>::Iterator it = m_groupsToDelete.begin(); it != m_groupsToDelete.end(); ++it) {
+    EntryGroupDict* dict = entryGroupDictByName(it->fieldName());
+    if(!dict) {
+      continue;
+    }
+    if(dict->count() == 1) {
+      m_entryGroupDicts.remove(it->fieldName());
+      m_entryGroups.remove(it->fieldName());
+    } else {
+      dict->remove(it->groupName());
+    }
+  }
+  m_groupsToDelete.clear();
+}
+
 // static
-int Collection::getID() {
-  static int id = 0;
+// merges values from e2 into e1
+bool Collection::mergeEntry(EntryPtr e1, EntryPtr e2, bool overwrite_) {
+  if(!e1 || !e2) {
+    return false;
+  }
+  bool ret1 = false;
+  FieldVec fields = e1->collection()->fields();
+  for(FieldVec::Iterator field = fields.begin(); field != fields.end(); ++field) {
+    if(e2->field(field).isEmpty()) {
+      continue;
+    }
+    if(overwrite_ || e1->field(field).isEmpty()) {
+//      myLog() << e1->title() << ": updating field(" << field->name() << ") to " << e2->field(field->name()) << endl;
+      e1->setField(field, e2->field(field));
+      ret1 = true;
+    }
+  }
+  // special case for tracks in albums
+  bool ret2 = false;
+  const QString sep = QString::fromLatin1("::");
+  if(e1->collection()->type() == Collection::Album) {
+    const QString track = QString::fromLatin1("track");
+    QStringList tracks1 = e1->fields(track, false);
+    QStringList tracks2 = e2->fields(track, false);
+    while(tracks1.count() < tracks2.count()) {
+      tracks1 << QString();
+    }
+    for(uint i = 0; i < tracks2.count(); ++i) {
+      if(tracks2[i].isEmpty()) {
+        continue;
+      }
+      if(tracks1[i].isEmpty()) {
+        tracks1[i] = tracks2[i];
+        ret2 = true;
+      } else {
+        QStringList parts1 = QStringList::split(sep, tracks1[i], true);
+        QStringList parts2 = QStringList::split(sep, tracks2[i], true);
+        bool addedPart = false;
+        while(parts1.count() < parts2.count()) {
+          parts1 += QString();
+        }
+        for(uint j = 1; j < parts2.count(); ++j) {
+          if(parts1[j].isEmpty()) {
+            parts1[j] = parts2[j];
+            addedPart = true;
+          }
+        }
+        if(addedPart) {
+          tracks1[i] = parts1.join(sep);
+          ret2 = true;
+        }
+      }
+    }
+    if(ret2) {
+      e1->setField(track, tracks1.join(QString::fromLatin1("; ")));
+    }
+  }
+  return ret1 || ret2;
+}
+
+long Collection::getID() {
+  static long id = 0;
   return ++id;
 }
 
