@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2001-2005 by Robby Stephenson
+    copyright            : (C) 2001-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -17,14 +17,27 @@
 #include "translators/bibtexhandler.h" // needed for BibtexHandler::cleanText()
 #include "document.h"
 #include "tellico_debug.h"
-
-#include <kdebug.h>
+#include "tellico_utils.h"
+#include "tellico_debug.h"
 
 #include <qregexp.h>
 
 using Tellico::Data::Entry;
+using Tellico::Data::EntryGroup;
 
-Entry::Entry(Collection* coll_) : KShared(), m_coll(coll_), m_id(-1) {
+EntryGroup::EntryGroup(const QString& group, const QString& field)
+   : EntryVec(), m_group(Tellico::shareString(group)), m_field(Tellico::shareString(field)) {
+}
+
+EntryGroup::~EntryGroup() {
+  // need a copy since we remove ourselves
+  EntryVec vec = *this;
+  for(Data::EntryVecIt entry = vec.begin(); entry != vec.end(); ++entry) {
+    entry->removeFromGroup(this);
+  }
+}
+
+Entry::Entry(CollPtr coll_) : KShared(), m_coll(coll_), m_id(-1) {
 #ifndef NDEBUG
   if(!coll_) {
     kdWarning() << "Entry() - null collection pointer!" << endl;
@@ -32,7 +45,7 @@ Entry::Entry(Collection* coll_) : KShared(), m_coll(coll_), m_id(-1) {
 #endif
 }
 
-Entry::Entry(Collection* coll_, int id_) : KShared(), m_coll(coll_), m_id(id_) {
+Entry::Entry(CollPtr coll_, int id_) : KShared(), m_coll(coll_), m_id(id_) {
 #ifndef NDEBUG
   if(!coll_) {
     kdWarning() << "Entry() - null collection pointer!" << endl;
@@ -48,30 +61,10 @@ Entry::Entry(const Entry& entry_) :
     m_formattedFields(entry_.m_formattedFields) {
 }
 
-Entry::Entry(const Entry& entry_, Collection* coll_) :
-    KShared(entry_),
-    m_coll(coll_),
-    m_id(-1),
-    m_fields(entry_.m_fields) {
-  // merge fields from old collection
-  Data::FieldVec fields = entry_.collection()->fields();
-  for(FieldVec::Iterator it = fields.begin(); it != fields.end(); ++it) {
-    // only add field if there's a value in the collection
-    if(!entry_.field(it->name()).isEmpty()) {
-      coll_->mergeField(it);
-    }
-  }
-  // special case for adding a book entry to a bibliography
-  if(m_coll->type() == Collection::Bibtex && entry_.collection()->type() == Collection::Book
-     && field(QString::fromLatin1("entry-type")).isEmpty()) {
-    setField(QString::fromLatin1("entry-type"), QString::fromLatin1("book"));
-  }
-}
-
 Entry& Entry::operator=(const Entry& other_) {
   if(this == &other_) return *this;
 
-//  kdDebug() << "Entry::operator=()" << endl;
+//  myDebug() << "Entry::operator=()" << endl;
   static_cast<KShared&>(*this) = static_cast<const KShared&>(other_);
   m_coll = other_.m_coll;
   m_id = other_.m_id;
@@ -83,21 +76,38 @@ Entry& Entry::operator=(const Entry& other_) {
 Entry::~Entry() {
 }
 
+Tellico::Data::CollPtr Entry::collection() const {
+  return m_coll;
+}
+
+void Entry::setCollection(CollPtr coll_) {
+  if(coll_ == m_coll) {
+    myDebug() << "Entry::setCollection() - already belongs to collection!" << endl;
+    return;
+  }
+  m_coll = coll_;
+  m_id = -1;
+}
+
 QString Entry::title() const {
   return formattedField(QString::fromLatin1("title"));
 }
 
+QString Entry::field(Data::FieldPtr field_, bool formatted_/*=false*/) const {
+  return field(field_->name(), formatted_);
+}
+
 QString Entry::field(const QString& fieldName_, bool formatted_/*=false*/) const {
   if(formatted_) {
     return formattedField(fieldName_);
   }
 
-  Field* f = m_coll->fieldByName(fieldName_);
+  FieldPtr f = m_coll->fieldByName(fieldName_);
   if(!f) {
     return QString::null;
   }
   if(f->type() == Field::Dependent) {
-    return dependentValue(f->description(), false);
+    return dependentValue(this, f->description(), false);
   }
 
   if(!m_fields.isEmpty() && m_fields.contains(fieldName_)) {
@@ -106,13 +116,17 @@ QString Entry::field(const QString& fieldName_, bool formatted_/*=false*/) const
   return QString::null;
 }
 
+QString Entry::formattedField(Data::FieldPtr field_) const {
+  return formattedField(field_->name());
+}
+
 QString Entry::formattedField(const QString& fieldName_) const {
-  Field* f = m_coll->fieldByName(fieldName_);
+  FieldPtr f = m_coll->fieldByName(fieldName_);
   if(!f) {
     return QString::null;
   }
   if(f->type() == Field::Dependent) {
-    return dependentValue(f->description(), true);
+    return dependentValue(this, f->description(), true);
   }
 
   Field::FormatFlag flag = f->formatFlag();
@@ -138,11 +152,31 @@ QString Entry::formattedField(const QString& fieldName_) const {
   return m_formattedFields[fieldName_];
 }
 
+QStringList Entry::fields(Data::FieldPtr field_, bool formatted_) const {
+  QString s = formatted_ ? formattedField(field_) : field(field_);
+  if(s.isEmpty()) {
+    return QStringList();
+  }
+  return QStringList::split(QString::fromLatin1("; "), s, true);
+}
+
 QStringList Entry::fields(const QString& field_, bool formatted_) const {
-  return QStringList::split(QString::fromLatin1("; "), formatted_ ? formattedField(field_) : field(field_));
+  QString s = formatted_ ? formattedField(field_) : field(field_);
+  if(s.isEmpty()) {
+    return QStringList();
+  }
+  return QStringList::split(QString::fromLatin1("; "), s, true);
+}
+
+bool Entry::setField(Data::FieldPtr field_, const QString& value_) {
+  return setField(field_->name(), value_);
 }
 
 bool Entry::setField(const QString& name_, const QString& value_) {
+  if(name_.isEmpty()) {
+    kdWarning() << "Entry::setField() - empty field name for value: " << value_ << endl;
+    return false;
+  }
   // an empty value means remove the field
   if(value_.isEmpty()) {
     if(!m_fields.isEmpty() && m_fields.contains(name_)) {
@@ -153,20 +187,32 @@ bool Entry::setField(const QString& name_, const QString& value_) {
   }
 
 #ifndef NDEBUG
-  if(m_coll && (m_coll->fields().count() == 0 || m_coll->fieldByName(name_) == 0)) {
-    kdDebug() << "Entry::setField() - unknown collection entry field - "
+  if(m_coll && (m_coll->fields().count() == 0 || !m_coll->hasField(name_))) {
+    myDebug() << "Entry::setField() - unknown collection entry field - "
               << name_ << endl;
     return false;
   }
 #endif
 
   if(m_coll && !m_coll->isAllowed(name_, value_)) {
-    kdDebug() << "Entry::setField() - for " << name_
+    myDebug() << "Entry::setField() - for " << name_
               << ", value is not allowed - " << value_ << endl;
     return false;
   }
 
-  m_fields.insert(name_, value_);
+  Data::FieldPtr f = m_coll->fieldByName(name_);
+  if(!f) {
+    return false;
+  }
+
+  // the string store is probable only useful for fields with auto-completion or choice/number/bool
+  if(!(f->flags() & Field::AllowMultiple) &&
+     ((f->type() == Field::Choice || f->type() == Field::Bool || f->type() == Field::Number) ||
+      (f->type() == Field::Line && (f->flags() & Field::AllowCompletion)))) {
+    m_fields.insert(Tellico::shareString(name_), Tellico::shareString(value_));
+  } else {
+    m_fields.insert(Tellico::shareString(name_), value_);
+  }
   invalidateFormattedFieldValue(name_);
   return true;
 }
@@ -176,7 +222,6 @@ bool Entry::addToGroup(EntryGroup* group_) {
     return false;
   }
 
-//  kdDebug() << "Entry::addToGroup() - adding group (" << group_->groupName() << ")" << endl;
   m_groups.push_back(group_);
   group_->append(this);
 //  m_coll->groupModified(group_);
@@ -187,22 +232,26 @@ bool Entry::removeFromGroup(EntryGroup* group_) {
   // if the removal isn't successful, just return
   bool success = m_groups.remove(group_);
   success = success && group_->remove(this);
-//  kdDebug() << "Entry::removeFromGroup() - removing from group - "
+//  myDebug() << "Entry::removeFromGroup() - removing from group - "
 //              << group_->fieldName() << "::" << group_->groupName() << endl;
   if(success) {
 //    m_coll->groupModified(group_);
   } else {
-    kdDebug() << "Entry::removeFromGroup() failed! " << endl;
+    myDebug() << "Entry::removeFromGroup() failed! " << endl;
   }
   return success;
 }
 
+void Entry::clearGroups() {
+  m_groups.clear();
+}
+
 // this function gets called before m_groups is updated. In fact, it is used to
 // update that list. This is the function that actually parses the field values
 // and returns the list of the group names.
 QStringList Entry::groupNamesByFieldName(const QString& fieldName_) const {
-//  kdDebug() << "Entry::groupsByfieldName() - " << fieldName_ << endl;
-  Field* f = m_coll->fieldByName(fieldName_);
+//  myDebug() << "Entry::groupsByfieldName() - " << fieldName_ << endl;
+  FieldPtr f = m_coll->fieldByName(fieldName_);
 
   // easy if not allowing multiple values
   if(!(f->flags() & Field::AllowMultiple)) {
@@ -216,19 +265,25 @@ QStringList Entry::groupNamesByFieldName(const QString& fieldName_) const {
 
   QStringList groups = fields(fieldName_, true);
   if(groups.isEmpty()) {
-    groups += Collection::s_emptyGroupTitle;
+    return Collection::s_emptyGroupTitle;
   } else if(f->type() == Field::Table) {
-    for(QStringList::Iterator it = groups.begin(); it != groups.end(); ++it) {
       // quick hack for tables, how often will a user have "::" in their value?
       // only use first column for group
+    QStringList::Iterator it = groups.begin();
+    while(it != groups.end()) {
       (*it) = (*it).section(QString::fromLatin1("::"),  0,  0);
+      if((*it).isEmpty()) {
+        it = groups.remove(it); // points to next in list
+      } else {
+        ++it;
+      }
     }
   }
   return groups;
 }
 
 bool Entry::isOwned() {
-  return (m_coll && m_coll->entryCount() > 0 && m_coll->entries().contains(this));
+  return (m_coll && m_id > -1 && m_coll->entryCount() > 0 && m_coll->entries().contains(this));
 }
 
 // a null string means invalidate all
@@ -241,9 +296,13 @@ void Entry::invalidateFormattedFieldValue(const QString& name_) {
 }
 
 // format is something like "%{year} %{author}"
-QString Entry::dependentValue(const QString& format_, bool autoCapitalize_) const {
+QString Entry::dependentValue(ConstEntryPtr entry_, const QString& format_, bool autoCapitalize_) {
+  if(!entry_) {
+    return format_;
+  }
+
   QString result, fieldName;
-  Field* f;
+  FieldPtr f;
 
   int endPos;
   int curPos = 0;
@@ -254,14 +313,14 @@ QString Entry::dependentValue(const QString& format_, bool autoCapitalize_) cons
       if(endPos > -1) {
         result += format_.mid(curPos, pctPos-curPos);
         fieldName = format_.mid(pctPos+2, endPos-pctPos-2);
-        f = m_coll->fieldByName(fieldName);
+        f = entry_->collection()->fieldByName(fieldName);
         if(!f) {
           // allow the user to also use field titles
-          f = m_coll->fieldByTitle(fieldName);
+          f = entry_->collection()->fieldByTitle(fieldName);
         }
         if(f) {
           // don't format, just capitalize
-          result += (autoCapitalize_ ? Field::capitalize(field(fieldName)) : field(fieldName));
+          result += (autoCapitalize_ ? Field::capitalize(entry_->field(f->name())) : entry_->field(f->name()));
         } else {
           result += format_.mid(pctPos, endPos-pctPos+1);
         }
@@ -276,6 +335,66 @@ QString Entry::dependentValue(const QString& format_, bool autoCapitalize_) cons
     pctPos = format_.find('%', curPos);
   }
   result += format_.mid(curPos, format_.length()-curPos);
-//  kdDebug() << "Entry::dependentValue() - " << format_ << " = " << result << endl;
-  return result;
+//  myDebug() << "Entry::dependentValue() - " << format_ << " = " << result << endl;
+  // sometimes field value might empty, resulting in multiple consecutive white spaces
+  // so let's simplify that...
+  return result.simplifyWhiteSpace();
+}
+
+int Entry::compareValues(EntryPtr e1, EntryPtr e2, const QString& f, ConstCollPtr c) {
+  return compareValues(e1, e2, c->fieldByName(f));
+}
+
+int Entry::compareValues(EntryPtr e1, EntryPtr e2, FieldPtr f) {
+  if(!e1 || !e2 || !f) {
+    return 0;
+  }
+  QString s1 = e1->field(f).lower();
+  QString s2 = e2->field(f).lower();
+  if(s1.isEmpty() || s2.isEmpty()) {
+    return 0;
+  }
+  // complicated string matching, here are the cases I want to match
+  // "bend it like beckham" == "bend it like beckham (widescreen edition)"
+  // "the return of the king" == "return of the king"
+  if(s1 == s2) {
+    return 5;
+  }
+  if(f->formatFlag() == Field::FormatName) {
+    s1 = e1->field(f, true).lower();
+    s2 = e2->field(f, true).lower();
+    if(s1 == s2) {
+      return 5;
+    }
+  }
+  Field::stripArticles(s1);
+  Field::stripArticles(s2);
+  if(s1 == s2) {
+//    myDebug() << "match without articles" << endl;
+    return 3;
+  }
+  QRegExp rx(QString::fromLatin1("\\s*\\(.*\\)\\s*"));
+  s1.remove(rx);
+  s2.remove(rx);
+  if(s1 == s2) {
+//    myDebug() << "match without parentheses" << endl;
+    return 1;
+  }
+  if(f->flags() & Field::AllowMultiple) {
+    QStringList sl1 = e1->fields(f, false);
+    QStringList sl2 = e2->fields(f, false);
+    int matches = 0;
+    for(QStringList::ConstIterator it = sl1.begin(); it != sl1.end(); ++it) {
+      matches += sl2.contains(*it);
+    }
+    if(matches == 0 && f->formatFlag() == Field::FormatName) {
+      sl1 = e1->fields(f, true);
+      sl2 = e2->fields(f, true);
+      for(QStringList::ConstIterator it = sl1.begin(); it != sl1.end(); ++it) {
+        matches += sl2.contains(*it);
+      }
+    }
+    return matches;
+  }
+  return 0;
 }
