@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2001-2004 by Robby Stephenson
+    copyright            : (C) 2001-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -12,17 +12,22 @@
  ***************************************************************************/
 
 #include "entryeditdialog.h"
-#include "tabcontrol.h"
+#include "gui/tabcontrol.h"
 #include "collection.h"
 #include "controller.h"
+#include "field.h"
+#include "entry.h"
+#include "tellico_utils.h"
+#include "tellico_kernel.h"
+#include "tellico_debug.h"
 
 #include <klocale.h>
-#include <kdebug.h>
 #include <kmessagebox.h>
-#include <kiconloader.h>
 #include <kaccelmanager.h>
+#include <kiconloader.h>
 #include <kdeversion.h>
-#include <kapplication.h> // for cursor
+#include <kpushbutton.h>
+#include <kaccel.h>
 
 #include <qlayout.h>
 #include <qstringlist.h>
@@ -31,29 +36,58 @@
 #include <qvbox.h>
 #include <qobjectlist.h>
 #include <qtabbar.h>
+#include <qstyle.h>
+#include <qapplication.h>
 
 namespace {
   // must be an even number
-  static const int NCOLS = 2; // number of columns of FieldWidgets
+  static const int NCOLS = 2; // number of columns of GUI::FieldWidgets
 }
 
 using Tellico::EntryEditDialog;
 
 EntryEditDialog::EntryEditDialog(QWidget* parent_, const char* name_)
-    : KDialogBase(parent_, name_, false, i18n("Edit Entry"), Help|User1|User2|Close, User1, false,
-                  KStdGuiItem::save(), KGuiItem(i18n("&New Entry"))),
+    : KDialogBase(parent_, name_, false, i18n("Edit Entry"), Help|User1|User2|User3|Apply|Close, User1, false,
+                  KGuiItem(i18n("&New Entry"))),
       m_currColl(0),
-      m_tabs(new TabControl(this)),
+      m_tabs(new GUI::TabControl(this)),
       m_modified(false),
-      m_isSaving(false) {
+      m_isOrphan(false),
+      m_isWorking(false) {
   setMainWidget(m_tabs);
 
-  enableButton(User1, false);
+  m_prevBtn = User3;
+  m_nextBtn = User2;
+  m_newBtn  = User1;
+  m_saveBtn = Apply;
+  KGuiItem save = KStdGuiItem::save();
+  save.setText(i18n("Sa&ve Entry"));
+  setButtonGuiItem(m_saveBtn, save);
+  enableButton(m_saveBtn, false);
 
-  connect(this, SIGNAL(user1Clicked()), SLOT(slotHandleSave()));
-  connect(this, SIGNAL(user2Clicked()), SLOT(slotHandleNew()));
+  connect(this, SIGNAL(applyClicked()), SLOT(slotHandleSave()));
+  connect(this, SIGNAL(user1Clicked()), SLOT(slotHandleNew()));
+  connect(this, SIGNAL(user2Clicked()), Controller::self(), SLOT(slotGoNextEntry()));
+  connect(this, SIGNAL(user3Clicked()), Controller::self(), SLOT(slotGoPrevEntry()));
 
-  m_isOrphan = false;
+  KGuiItem prev;
+  prev.setIconName(QString::fromLatin1(QApplication::reverseLayout() ? "forward" : "back"));
+  prev.setToolTip(i18n("Go to the previous entry in the collection"));
+  prev.setWhatsThis(prev.toolTip());
+
+  KGuiItem next;
+  next.setIconName(QString::fromLatin1(QApplication::reverseLayout() ? "back" : "forward"));
+  next.setToolTip(i18n("Go to the next entry in the collection"));
+  next.setWhatsThis(next.toolTip());
+
+  setButtonGuiItem(m_nextBtn, next);
+  setButtonGuiItem(m_prevBtn, prev);
+
+  KAccel* accel = new KAccel(this);
+  accel->insert(QString::fromLatin1("Go Prev"), QString(), prev.toolTip(), Qt::Key_PageUp,
+                Controller::self(), SLOT(slotGoPrevEntry()));
+  accel->insert(QString::fromLatin1("Go Next"), QString(), next.toolTip(), Qt::Key_PageDown,
+                Controller::self(), SLOT(slotGoNextEntry()));
 
   setHelp(QString::fromLatin1("entry-editor"));
 
@@ -72,44 +106,50 @@ void EntryEditDialog::slotClose() {
 }
 
 void EntryEditDialog::slotReset() {
-//  kdDebug() << "EntryEditDialog::slotReset()" << endl;
-
-  enableButton(User1, false);
-  setButtonText(User1, i18n("Sa&ve Entry"));
-  m_currColl = 0;
-  m_currEntries.clear();;
-
-  m_modified = false;
-
-  m_tabs->clear(); // FieldWidgets get deleted here
-  m_widgetDict.clear();
-}
-
-void EntryEditDialog::setLayout(Data::Collection* coll_) {
-  if(!coll_) {
+//  myDebug() << "EntryEditDialog::slotReset()" << endl;
+  if(m_isWorking) {
     return;
   }
 
-//  kdDebug() << "EntryEditDialog::setLayout()" << endl;
+  enableButton(m_saveBtn, false);
+  setButtonText(m_saveBtn, i18n("Sa&ve Entry"));
+  m_currColl = 0;
+  m_currEntries.clear();
 
-  actionButton(User2)->setIconSet(KGlobal::iconLoader()->loadIcon(coll_->entryName(), KIcon::User));
+  m_modified = false;
+
+  m_tabs->clear(); // GUI::FieldWidgets get deleted here
+  m_widgetDict.clear();
+}
+
+void EntryEditDialog::setLayout(Data::CollPtr coll_) {
+  if(!coll_ || m_isWorking) {
+    return;
+  }
+//  myDebug() << "EntryEditDialog::setLayout()" << endl;
+
+  actionButton(m_newBtn)->setIconSet(UserIconSet(coll_->typeName()));
 
   setUpdatesEnabled(false);
   if(m_tabs->count() > 0) {
-//    kdDebug() << "EntryEditDialog::setLayout() - resetting contents." << endl;
+//    myDebug() << "EntryEditDialog::setLayout() - resetting contents." << endl;
     slotReset();
   }
+  m_isWorking = true;
 
   m_currColl = coll_;
-  FieldWidget::s_coll = coll_;
 
   int maxHeight = 0;
   QPtrList<QWidget> gridList;
   bool noChoices = true;
 
+  bool focusedFirst = false;
   QStringList catList = m_currColl->fieldCategories();
   for(QStringList::ConstIterator catIt = catList.begin(); catIt != catList.end(); ++catIt) {
-    Data::FieldList list = m_currColl->fieldsByCategory(*catIt);
+    Data::FieldVec fields = m_currColl->fieldsByCategory(*catIt);
+    if(fields.isEmpty()) { // sanity check
+      continue;
+    }
 
     // if this layout model is changed, be sure to check slotUpdateField()
     QWidget* page = new QWidget(m_tabs);
@@ -119,12 +159,17 @@ void EntryEditDialog::setLayout(Data::Collection* coll_) {
     QWidget* grid = new QWidget(page);
     gridList.append(grid);
     // (parent, nrows, ncols, margin, spacing)
-    // spacing gets a bit weird, if there are absolute no Choice fields, then spacing should be 4
-    QGridLayout* layout = new QGridLayout(grid, 0, NCOLS, 8, 0);
+    // spacing gets a bit weird, if there are absolutely no Choice fields,
+    // then spacing should be 5, which is set later
+    QGridLayout* layout = new QGridLayout(grid, 0, NCOLS, 8, 2);
+    // keramik styles make big widget, cut down the spacing a bit
+    if(QCString(style().name()).lower().find("keramik", 0, false) > -1) {
+      layout->setSpacing(0);
+    }
 
     boxLayout->addWidget(grid, 0);
     // those with multiple, get a stretch
-    if(list.count() > 1 || !list.getFirst()->isSingleCategory()) {
+    if(fields.count() > 1 || !fields[0]->isSingleCategory()) {
       boxLayout->addStretch(1);
     }
 
@@ -132,40 +177,49 @@ void EntryEditDialog::setLayout(Data::Collection* coll_) {
     QValueVector<bool> expands(NCOLS, false);
     QValueVector<int> maxWidth(NCOLS, 0);
 
-    Data::FieldListIterator it(list); // needed later
-    for(int count = 0; it.current(); ++it) {
-      Data::Field* f = it.current();
+    Data::FieldVec::Iterator it = fields.begin(); // needed later
+    for(int count = 0; it != fields.end(); ++it) {
+      Data::FieldPtr field = it;
       // ReadOnly and Dependent fields don't get widgets
-      if(f->type() == Data::Field::ReadOnly || f->type() == Data::Field::Dependent) {
+      if(field->type() == Data::Field::ReadOnly || field->type() == Data::Field::Dependent) {
         continue;
       }
-      if(f->type() == Data::Field::Choice) {
+      if(field->type() == Data::Field::Choice) {
         noChoices = false;
       }
 
-      FieldWidget* widget = new FieldWidget(f, grid);
+      GUI::FieldWidget* widget = GUI::FieldWidget::create(field, grid);
+      if(!widget) {
+        continue;
+      }
       connect(widget, SIGNAL(modified()), SLOT(slotSetModified()));
+      if(!focusedFirst && widget->isFocusEnabled()) {
+        widget->setFocus();
+        focusedFirst = true;
+      }
 
-      layout->addWidget(widget, count/NCOLS, count%NCOLS);
-      layout->setRowStretch(count/NCOLS, 1);
+      int r = count/NCOLS;
+      int c = count%NCOLS;
+      layout->addWidget(widget, r, c);
+      layout->setRowStretch(r, 1);
 
-      m_widgetDict.insert(QString::number(m_currColl->id()) + f->name(), widget);
+      m_widgetDict.insert(QString::number(m_currColl->id()) + field->name(), widget);
 
       maxWidth[count%NCOLS] = KMAX(maxWidth[count%NCOLS], widget->labelWidth());
       if(widget->expands()) {
         expands[count%NCOLS] = true;
       }
       widget->updateGeometry();
-      if(!f->isSingleCategory()) {
+      if(!field->isSingleCategory()) {
         maxHeight = KMAX(maxHeight, widget->minimumSizeHint().height());
       }
       ++count;
     }
 
     // now, the labels in a column should all be the same width
-    it.toFirst();
-    for(int count = 0; it.current(); ++it) {
-      FieldWidget* widget = m_widgetDict.find(QString::number(m_currColl->id()) + it.current()->name());
+    it = fields.begin();
+    for(int count = 0; it != fields.end(); ++it) {
+      GUI::FieldWidget* widget = m_widgetDict.find(QString::number(m_currColl->id()) + it->name());
       if(widget) {
         widget->setLabelWidth(maxWidth[count%NCOLS]);
         ++count;
@@ -207,15 +261,12 @@ void EntryEditDialog::setLayout(Data::Collection* coll_) {
   m_tabs->setMinimumWidth(m_tabs->sizeHint().width());
 
   // update keyboard accels
-  // only want to manage tabBar(), but KDE bug 71769 means the parent widget must be used
-#if KDE_IS_VERSION(3,2,90)
+  // only want to manage tabBar()
   KAcceleratorManager::manage(m_tabs->tabBar());
-#else
-  KAcceleratorManager::manage(m_tabs->tabBar()->parentWidget());
-#endif
 
   m_tabs->setCurrentPage(0);
 
+  m_isWorking = false;
   slotHandleNew();
   m_modified = false; // because the year is inserted
 }
@@ -224,38 +275,31 @@ void EntryEditDialog::slotHandleNew() {
   if(!m_currColl || !queryModified()) {
     return;
   }
-//  kdDebug() << "EntryEditDialog::slotHandleNew()" << endl;
 
-  m_tabs->setResetFocus(true);
   m_tabs->setCurrentPage(0);
+  m_tabs->setFocusToFirstChild();
   clear();
-  Controller::self()->slotUpdateSelection(this); // FIXME:should have a clearSelection() function
+  m_isWorking = true; // clear() will get called again
+  Controller::self()->slotClearSelection();
+  m_isWorking = false;
 
-  Data::Entry* entry = new Data::Entry(m_currColl);
+  enableButton(m_prevBtn, false);
+  enableButton(m_nextBtn, false);
+
+  Data::EntryPtr entry = new Data::Entry(m_currColl);
   m_currEntries.append(entry);
   m_isOrphan = true;
-
-// special case for purchase date, insert current year
-//  QString key = QString::number(m_currColl->id()) + QString::fromLatin1("pur_date");
-//  FieldWidget* widget = m_widgetDict.find(key);
-//  if(widget) {
-//    QDate now = QDate::currentDate();
-//    widget->setText(QString::number(now.year()));
-//    m_modified = false;
-//  }
 }
 
 void EntryEditDialog::slotHandleSave() {
-//  kdDebug() << "EntryEditDialog::slotHandleSave()" << endl;
-  if(!m_currColl) {
-    // big problem
-    kdDebug() << "EntryEditDialog::slotHandleSave() - no valid collection pointer" << endl;
+  if(!m_currColl || m_isWorking) {
     return;
   }
 
-  m_isSaving = true;
+  m_isWorking = true;
 
   if(m_currEntries.isEmpty()) {
+    myDebug() << "EntryEditDialog::slotHandleSave() - creating new entry" << endl;
     m_currEntries.append(new Data::Entry(m_currColl));
     m_isOrphan = true;
   }
@@ -263,61 +307,75 @@ void EntryEditDialog::slotHandleSave() {
   // add a message box if multiple items are selected
   if(m_currEntries.count() > 1) {
     QStringList names;
-    for(Data::EntryListIterator entryIt(m_currEntries); entryIt.current(); ++entryIt) {
-      names += entryIt.current()->title();
+    for(Data::EntryVec::ConstIterator entry = m_currEntries.constBegin(); entry != m_currEntries.constEnd(); ++entry) {
+      names += entry->title();
     }
     QString str(i18n("Do you really want to modify these entries?"));
     QString dontAsk = QString::fromLatin1("SaveMultipleBooks"); // don't change 'books', invisible anyway
-    int ret = KMessageBox::questionYesNoList(this, str, names, i18n("Modify Multiple Entries?"),
+    int ret = KMessageBox::questionYesNoList(this, str, names, i18n("Modify Multiple Entries"),
                                              KStdGuiItem::yes(), KStdGuiItem::no(), dontAsk);
     if(ret != KMessageBox::Yes) {
+      m_isWorking = false;
       return;
     }
   }
 
-  kapp->setOverrideCursor(Qt::waitCursor);
-  Data::FieldListIterator fIt(m_currColl->fieldList());
-  // boolean to keep track if every possible field is empty
-  bool empty = true;
-  for(Data::EntryListIterator entryIt(m_currEntries); entryIt.current(); ++entryIt) {
-    for(fIt.toFirst(); fIt.current(); ++fIt) {
-      QString key = QString::number(m_currColl->id()) + fIt.current()->name();
-      FieldWidget* widget = m_widgetDict.find(key);
+  GUI::CursorSaver cs(Qt::waitCursor);
+
+  Data::EntryVec oldEntries;
+  Data::FieldVec fields = m_currColl->fields();
+  // boolean to keep track if any field gets changed
+  bool modified = false;
+  for(Data::EntryVecIt entry = m_currEntries.begin(); entry != m_currEntries.end(); ++entry) {
+    // if the entry is owned, then we're modifying an existing entry, keep a copy of the old one
+    if(entry->isOwned()) {
+      oldEntries.append(new Data::Entry(*entry));
+    }
+    for(Data::FieldVec::Iterator fIt = fields.begin(); fIt != fields.end(); ++fIt) {
+      QString key = QString::number(m_currColl->id()) + fIt->name();
+      GUI::FieldWidget* widget = m_widgetDict.find(key);
       if(widget && widget->isEnabled()) {
         QString temp = widget->text();
         // ok to set field empty string, just not all of them
-        entryIt.current()->setField(fIt.current()->name(), temp);
-        if(!temp.isEmpty()) {
-          empty = false;
+        if(modified == false && entry->field(fIt) != temp) {
+          modified = true;
         }
+        entry->setField(fIt, temp);
       }
     }
   }
 
   // if something was not empty, signal a save
-  if(!empty) {
+  if(modified) {
     m_isOrphan = false;
-    emit signalSaveEntries(m_currEntries);
-    if(!m_currEntries.getFirst()->title().isEmpty()) {
-      setCaption(i18n("Edit Entry") + QString::fromLatin1(" - ") + m_currEntries.getFirst()->title());
+    if(oldEntries.isEmpty()) {
+      Kernel::self()->addEntries(m_currEntries, false);
+    } else {
+      Kernel::self()->modifyEntries(oldEntries, m_currEntries);
+    }
+    if(!m_currEntries.isEmpty() && !m_currEntries[0]->title().isEmpty()) {
+      setCaption(i18n("Edit Entry") + QString::fromLatin1(" - ") + m_currEntries[0]->title());
     }
   }
 
-
-  kapp->restoreOverrideCursor();
   m_modified = false;
-  m_isSaving = false;
-  enableButton(User1, false);
+  m_isWorking = false;
+  enableButton(m_saveBtn, false);
 //  slotHandleNew();
 }
 
 void EntryEditDialog::clear() {
-//  kdDebug() << "EntryEditDialog::clear()" << endl;
+//  myDebug() << "EntryEditDialog::clear()" << endl;
+  if(m_isWorking) {
+    return;
+  }
 
+  m_isWorking = true;
   // clear the widgets
-  for(QDictIterator<FieldWidget> it(m_widgetDict); it.current(); ++it) {
+  for(QDictIterator<GUI::FieldWidget> it(m_widgetDict); it.current(); ++it) {
     it.current()->setEnabled(true);
     it.current()->clear();
+    it.current()->insertDefault();
   }
 
   setCaption(i18n("Edit Entry"));
@@ -326,25 +384,25 @@ void EntryEditDialog::clear() {
     if(m_currEntries.count() > 1) {
       kdWarning() << "EntryEditDialog::clear() - is an orphan, but more than one" << endl;
     }
-    delete m_currEntries.getFirst();
     m_isOrphan = false;
   }
   m_currEntries.clear();
 
-  setButtonText(User1, i18n("Sa&ve Entry"));
-  enableButton(User1, false);
+  setButtonText(m_saveBtn, i18n("Sa&ve Entry"));
+  enableButton(m_saveBtn, false);
 
   m_modified = false;
+  m_isWorking = false;
 }
 
-void EntryEditDialog::setContents(const Data::EntryList& list_) {
+void EntryEditDialog::setContents(Data::EntryVec entries_) {
   // this slot might get called if we try to save multiple items, so just return
-  if(m_isSaving) {
+  if(m_isWorking) {
     return;
   }
 
-  if(list_.isEmpty()) {
-//    kdDebug() << "EntryEditDialog::setContents() - empty list" << endl;
+  if(entries_.isEmpty()) {
+//    myDebug() << "EntryEditDialog::setContents() - empty list" << endl;
     if(queryModified()) {
       blockSignals(true);
       slotHandleNew();
@@ -353,77 +411,77 @@ void EntryEditDialog::setContents(const Data::EntryList& list_) {
     return;
   }
 
-//  kdDebug() << "EntryEditDialog::setContents() - " << list_.count() << " entries" << endl;
+//  myDebug() << "EntryEditDialog::setContents() - " << entries_.count() << " entries" << endl;
 
   // first set contents to first item
-  setContents(list_.getFirst());
+  setContents(entries_.front());
   // something weird...if list count can actually be 1 before the setContents call
   // and 0 after it. Why is that? It's const!
-  if(list_.count() < 2) {
+  if(entries_.count() < 2) {
     return;
   }
 
   // multiple entries, so don't set caption
-  setCaption(i18n("Edit Entry"));
-
-  // keep track of the entries
-  m_currEntries = list_;
+  setCaption(i18n("Edit Entries"));
 
+  m_currEntries = entries_;
+  m_isWorking = true;
   blockSignals(true);
 
-  Data::EntryListIterator entryIt(list_);
-  for(Data::FieldListIterator fIt(m_currColl->fieldList()); fIt.current(); ++fIt) {
-    QString key = QString::number(m_currColl->id()) + fIt.current()->name();
-    FieldWidget* widget = m_widgetDict.find(key);
+  Data::EntryVec::ConstIterator entry;
+  Data::FieldVec fields = m_currColl->fields();
+  for(Data::FieldVec::Iterator fIt = fields.begin(); fIt != fields.end(); ++fIt) {
+    QString key = QString::number(m_currColl->id()) + fIt->name();
+    GUI::FieldWidget* widget = m_widgetDict.find(key);
     if(!widget) { // probably read-only
       continue;
     }
     widget->editMultiple(true);
 
-    QString value = list_.getFirst()->field(fIt.current()->name());
-    for(++entryIt; entryIt.current(); ++entryIt) { // skip checking the first one
-      if(entryIt.current()->field(fIt.current()->name()) != value) {
+    QString value = entries_[0]->field(fIt);
+    entry = entries_.constBegin();
+    for(++entry; entry != entries_.constEnd(); ++entry) { // skip checking the first one
+      if(entry->field(fIt) != value) {
         widget->setEnabled(false);
         break;
       }
     }
-    entryIt.toFirst();
   } // end field loop
 
   blockSignals(false);
+  m_isWorking = false;
 
-  setButtonText(User1, i18n("Sa&ve Entries"));
+  // can't go to next entry if multiple are selected
+  enableButton(m_prevBtn, false);
+  enableButton(m_nextBtn, false);
+  setButtonText(m_saveBtn, i18n("Sa&ve Entries"));
 }
 
-void EntryEditDialog::setContents(Data::Entry* entry_, const QString& highlight_/*empty*/) {
+void EntryEditDialog::setContents(Data::EntryPtr entry_) {
   bool ok = queryModified();
-  if(!ok) {
+  if(!ok || m_isWorking) {
     return;
   }
 
   if(!entry_) {
-    kdDebug() << "EntryEditDialog::setContents() - null entry pointer" << endl;
+    myDebug() << "EntryEditDialog::setContents() - null entry pointer" << endl;
     slotHandleNew();
     return;
   }
 
-  m_tabs->setResetFocus(false);
-
- // kdDebug() << "EntryEditDialog::setContents() - " << entry_->title() << endl;
+//  myDebug() << "EntryEditDialog::setContents() - " << entry_->title() << endl;
   blockSignals(true);
   clear();
   blockSignals(false);
+  m_isWorking = true;
   m_currEntries.append(entry_);
 
   if(!entry_->title().isEmpty()) {
     setCaption(i18n("Edit Entry") + QString::fromLatin1(" - ") + entry_->title());
   }
 
-  // I'll end up nulling the highlight string, so make a copy
-  QString highlight = highlight_;
-
   if(m_currColl != entry_->collection()) {
-    kdDebug() << "EntryEditDialog::setContents() - collections don't match" << endl;
+    myDebug() << "EntryEditDialog::setContents() - collections don't match" << endl;
     m_currColl = entry_->collection();
   }
 
@@ -431,52 +489,46 @@ void EntryEditDialog::setContents(Data::Entry* entry_, const QString& highlight_
 
   slotSetModified(false);
 
-  int highlightPage = -1;
-  for(Data::FieldListIterator it(m_currColl->fieldList()); it.current(); ++it) {
-    QString key = QString::number(m_currColl->id()) + it.current()->name();
-    FieldWidget* widget = m_widgetDict.find(key);
+  Data::FieldVec fields = m_currColl->fields();
+  for(Data::FieldVec::Iterator field = fields.begin(); field != fields.end(); ++field) {
+    QString key = QString::number(m_currColl->id()) + field->name();
+    GUI::FieldWidget* widget = m_widgetDict.find(key);
     if(!widget) { // is probably read-only
       continue;
     }
 
-    QString value = entry_->field(it.current()->name());
+    QString value = entry_->field(field);
     widget->setText(value);
-    if(!highlight.isEmpty() && value.lower().contains(highlight.lower())) {
-      widget->setHighlighted(highlight);
-      // now I only want to worry about first string found, so empty it
-      highlight = QString::null;
-      highlightPage = m_tabs->indexOf(widget->parentWidget());
-    }
     widget->setEnabled(true);
     widget->editMultiple(false);
   } // end field loop
 
+  enableButton(m_prevBtn, true);
+  enableButton(m_nextBtn, true);
   if(entry_->isOwned()) {
-    setButtonText(User1, i18n("Sa&ve Entry"));
-    enableButton(User1, m_modified);
+    setButtonText(m_saveBtn, i18n("Sa&ve Entry"));
+    enableButton(m_saveBtn, m_modified);
   } else {
     slotSetModified(true);
   }
-  if(highlightPage > -1) {
-    m_tabs->setCurrentPage(highlightPage);
-  }
+  m_isWorking = false;
 }
 
-void EntryEditDialog::removeField(Data::Field* field_) {
+void EntryEditDialog::removeField(Data::CollPtr, Data::FieldPtr field_) {
   if(!field_) {
     return;
   }
 
-//  kdDebug() << "EntryEditDialog::removeField - name = " << field_->name() << endl;
+//  myDebug() << "EntryEditDialog::removeField - name = " << field_->name() << endl;
   QString key = QString::number(m_currColl->id()) + field_->name();
-  FieldWidget* widget = m_widgetDict.find(key);
+  GUI::FieldWidget* widget = m_widgetDict.find(key);
   if(widget) {
     m_widgetDict.remove(key);
     // if this is the last field in the category, need to remove the tab page
     // this function is called after the field has been removed from the collection,
     // so the category should be gone from the category list
     if(m_currColl->fieldCategories().findIndex(field_->category()) == -1) {
-//      kdDebug() << "last field in the category" << endl;
+//      myDebug() << "last field in the category" << endl;
       // fragile, widget's parent is the grid, whose parent is the tab page
       QWidget* w = widget->parentWidget()->parentWidget();
       m_tabs->removePage(w);
@@ -489,10 +541,10 @@ void EntryEditDialog::removeField(Data::Field* field_) {
       QValueVector<bool> expands(NCOLS, false);
       QValueVector<int> maxWidth(NCOLS, 0);
 
-      Data::FieldList list = m_currColl->fieldsByCategory(field_->category());
-      Data::FieldListIterator it(list);
-      for(int count = 0; it.current(); ++it) {
-        FieldWidget* widget = m_widgetDict.find(QString::number(m_currColl->id()) + it.current()->name());
+      Data::FieldVec vec = m_currColl->fieldsByCategory(field_->category());
+      Data::FieldVec::Iterator it = vec.begin();
+      for(int count = 0; it != vec.end(); ++it) {
+        GUI::FieldWidget* widget = m_widgetDict.find(QString::number(m_currColl->id()) + it->name());
         if(widget) {
           layout->remove(widget);
           layout->addWidget(widget, count/NCOLS, count%NCOLS);
@@ -507,9 +559,9 @@ void EntryEditDialog::removeField(Data::Field* field_) {
       }
 
       // now, the labels in a column should all be the same width
-      it.toFirst();
-      for(int count = 0; it.current(); ++it) {
-        FieldWidget* widget = m_widgetDict.find(QString::number(m_currColl->id()) + it.current()->name());
+      it = vec.begin();
+      for(int count = 0; it != vec.end(); ++it) {
+        GUI::FieldWidget* widget = m_widgetDict.find(QString::number(m_currColl->id()) + it->name());
         if(widget) {
           widget->setLabelWidth(maxWidth[count%NCOLS]);
           ++count;
@@ -526,46 +578,56 @@ void EntryEditDialog::removeField(Data::Field* field_) {
   }
 }
 
-void EntryEditDialog::slotUpdateCompletions(Data::Entry* entry_) {
+void EntryEditDialog::updateCompletions(Data::EntryPtr entry_) {
 #ifndef NDEBUG
   if(m_currColl != entry_->collection()) {
-    kdDebug() << "EntryEditDialog::slotUpdateCompletions - inconsistent collection pointers!" << endl;
-    m_currColl = entry_->collection();
+    myDebug() << "EntryEditDialog::updateCompletions - inconsistent collection pointers!" << endl;
+//    m_currColl = entry_->collection();
   }
 #endif
 
-  for(Data::FieldListIterator it(m_currColl->fieldList()); it.current(); ++it) {
-    if(it.current()->type() == Data::Field::Line
-       && it.current()->flags() & Data::Field::AllowCompletion) {
-      QString key = QString::number(m_currColl->id()) + it.current()->name();
-      FieldWidget* widget = m_widgetDict.find(key);
-      if(widget) {
-        if(it.current()->flags() & Data::Field::AllowMultiple) {
-          QStringList items = entry_->fields(it.current()->name(), false);
-          for(QStringList::ConstIterator it = items.begin(); it != items.end(); ++it) {
-            widget->addCompletionObjectItem(*it);
-          }
-        } else {
-          widget->addCompletionObjectItem(entry_->field(it.current()->name()));
-        }
+  Data::FieldVec fields = m_currColl->fields();
+  for(Data::FieldVec::Iterator it = fields.begin(); it != fields.end(); ++it) {
+    if(it->type() != Data::Field::Line
+       || !(it->flags() & Data::Field::AllowCompletion)) {
+      continue;
+    }
+
+    QString key = QString::number(m_currColl->id()) + it->name();
+    GUI::FieldWidget* widget = m_widgetDict.find(key);
+    if(!widget) {
+      continue;
+    }
+    if(it->flags() & Data::Field::AllowMultiple) {
+      QStringList items = entry_->fields(it, false);
+      for(QStringList::ConstIterator it = items.begin(); it != items.end(); ++it) {
+        widget->addCompletionObjectItem(*it);
       }
+    } else {
+      widget->addCompletionObjectItem(entry_->field(it->name()));
     }
   }
 }
 
 void EntryEditDialog::slotSetModified(bool mod_/*=true*/) {
   m_modified = mod_;
-  enableButton(User1, mod_);
+  enableButton(m_saveBtn, mod_);
 }
 
 bool EntryEditDialog::queryModified() {
-//  kdDebug() << "EntryEditDialog::queryModified() - modified is " << (m_modified?"true":"false") << endl;
+//  myDebug() << "EntryEditDialog::queryModified() - modified is " << (m_modified?"true":"false") << endl;
   bool ok = true;
+  // assume that if the dialog is hidden, we shouldn't ask the user to modify changes
+  if(!isShown()) {
+    m_modified = false;
+  }
   if(m_modified) {
     QString str(i18n("The current entry has been modified.\n"
                       "Do you want to enter the changes?"));
-    int want_save = KMessageBox::warningYesNoCancel(this, str, i18n("Warning!"),
-                                                    i18n("Save Entry"), KStdGuiItem::discard());
+    KGuiItem item = KStdGuiItem::save();
+    item.setText(i18n("Save Entry"));
+    int want_save = KMessageBox::warningYesNoCancel(this, str, i18n("Unsaved Changes"),
+                                                    item, KStdGuiItem::discard());
     switch(want_save) {
       case KMessageBox::Yes:
         slotHandleSave();
@@ -586,11 +648,11 @@ bool EntryEditDialog::queryModified() {
 }
 
 // modified fields will always have the same name
-void EntryEditDialog::slotUpdateField(Data::Collection* coll_, Data::Field* newField_, Data::Field* oldField_) {
-//  kdDebug() << "EntryEditDialog::slotUpdateField() - " << newField_->name() << endl;
+void EntryEditDialog::modifyField(Data::CollPtr coll_, Data::FieldPtr oldField_, Data::FieldPtr newField_) {
+//  myDebug() << "EntryEditDialog::slotUpdateField() - " << newField_->name() << endl;
 
   if(coll_ != m_currColl) {
-    kdDebug() << "EntryEditDialog::slotUpdateField() - wrong collection pointer!" << endl;
+    myDebug() << "EntryEditDialog::slotUpdateField() - wrong collection pointer!" << endl;
     m_currColl = coll_;
   }
 
@@ -598,28 +660,27 @@ void EntryEditDialog::slotUpdateField(Data::Collection* coll_, Data::Field* newF
   // also if the category changed for a non-single field, since a new tab must be created
   if(oldField_->type() != newField_->type()
      || (oldField_->category() != newField_->category() && !newField_->isSingleCategory())) {
-    Data::EntryList entries = m_currEntries;
     bool modified = m_modified;
     setLayout(coll_);
-    setContents(entries);
+    setContents(m_currEntries);
     m_modified = modified;
     return;
   }
 
   QString key = QString::number(coll_->id()) + oldField_->name();
-  FieldWidget* widget = m_widgetDict[key];
+  GUI::FieldWidget* widget = m_widgetDict[key];
   if(widget) {
-    widget->updateField(newField_, oldField_);
+    widget->updateField(oldField_, newField_);
     // need to update label widths
     if(newField_->title() != oldField_->title()) {
       int maxWidth = 0;
-      QObjectList* childList = widget->parentWidget()->queryList("Tellico::FieldWidget", 0, false, false);
+      QObjectList* childList = widget->parentWidget()->queryList("Tellico::GUI::FieldWidget", 0, false, false);
       QObjectListIt it(*childList);
       for(it.toFirst(); it.current(); ++it) {
-        maxWidth = KMAX(maxWidth, static_cast<FieldWidget*>(it.current())->labelWidth());
+        maxWidth = KMAX(maxWidth, static_cast<GUI::FieldWidget*>(it.current())->labelWidth());
       }
       for(it.toFirst(); it.current(); ++it) {
-        static_cast<FieldWidget*>(it.current())->setLabelWidth(maxWidth);
+        static_cast<GUI::FieldWidget*>(it.current())->setLabelWidth(maxWidth);
       }
       delete childList;
     }
@@ -632,4 +693,23 @@ void EntryEditDialog::slotUpdateField(Data::Collection* coll_, Data::Field* newF
   }
 }
 
+void EntryEditDialog::addEntries(Data::EntryVec entries_) {
+  for(Data::EntryVecIt entry = entries_.begin(); entry != entries_.end(); ++entry) {
+    updateCompletions(entry);
+  }
+}
+
+void EntryEditDialog::modifyEntries(Data::EntryVec entries_) {
+  bool updateContents = false;
+  for(Data::EntryVecIt entry = entries_.begin(); entry != entries_.end(); ++entry) {
+    updateCompletions(entry);
+    if(!updateContents && m_currEntries.contains(entry)) {
+      updateContents = true;
+    }
+  }
+  if(updateContents) {
+    setContents(m_currEntries);
+  }
+}
+
 #include "entryeditdialog.moc"
