@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2005 by Robby Stephenson
+    copyright            : (C) 2005-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -15,7 +15,6 @@
 #include "../tellico_kernel.h"
 #include "../latin1literal.h"
 #include "../collection.h"
-#include "../document.h"
 #include "../entry.h"
 #include "../filehandler.h"
 #include "../translators/xslthandler.h"
@@ -23,12 +22,12 @@
 
 #include <klocale.h>
 #include <kconfig.h>
-#include <kglobal.h> // for KMIN
 #include <kstandarddirs.h>
 
 #include <qdom.h>
 #include <qlabel.h>
 #include <qlayout.h>
+#include <qfile.h>
 
 //#define ENTREZ_TEST
 
@@ -38,6 +37,7 @@ namespace {
   static const char* ENTREZ_SEARCH_CGI = "esearch.fcgi";
   static const char* ENTREZ_SUMMARY_CGI = "esummary.fcgi";
   static const char* ENTREZ_FETCH_CGI = "efetch.fcgi";
+  static const char* ENTREZ_LINK_CGI = "elink.fcgi";
   static const char* ENTREZ_DEFAULT_DATABASE = "pubmed";
 }
 
@@ -45,37 +45,38 @@ using Tellico::Fetch::EntrezFetcher;
 Tellico::XSLTHandler* EntrezFetcher::s_xsltHandler = 0;
 
 EntrezFetcher::EntrezFetcher(QObject* parent_, const char* name_) : Fetcher(parent_, name_),
-    m_name(i18n("Entrez Database")), m_step(Begin), m_started(false) {
+    m_step(Begin), m_started(false) {
 }
 
 EntrezFetcher::~EntrezFetcher() {
 }
 
+QString EntrezFetcher::defaultName() {
+  return i18n("Entrez Database");
+}
+
 QString EntrezFetcher::source() const {
-  return m_name;
+  return m_name.isEmpty() ? defaultName() : m_name;
 }
 
 bool EntrezFetcher::canFetch(int type) const {
   return type == Data::Collection::Bibtex;
 }
 
-void EntrezFetcher::readConfig(KConfig* config_, const QString& group_) {
+void EntrezFetcher::readConfigHook(KConfig* config_, const QString& group_) {
   KConfigGroupSaver groupSaver(config_, group_);
-  QString s = config_->readEntry("Name");
-  if(!s.isEmpty()) {
-    m_name = s;
-  }
-  s = config_->readEntry("Database", QString::fromLatin1("pubmed")); // default to pubmed
+  QString s = config_->readEntry("Database", QString::fromLatin1("pubmed")); // default to pubmed
   if(!s.isEmpty()) {
     m_dbname = s;
   }
+  m_fields = config_->readListEntry("Custom Fields");
 }
 
-void EntrezFetcher::search(FetchKey key_, const QString& value_, bool) {
+void EntrezFetcher::search(FetchKey key_, const QString& value_) {
   m_started = true;
 // only search if current collection is a bibliography
   if(Kernel::self()->collectionType() != Data::Collection::Bibtex) {
-    kdDebug() << "EntrezFetcher::search() - collection type mismatch, stopping" << endl;
+    myDebug() << "EntrezFetcher::search() - collection type mismatch, stopping" << endl;
     stop();
     return;
   }
@@ -124,7 +125,7 @@ void EntrezFetcher::search(FetchKey key_, const QString& value_, bool) {
 #endif
 
   m_step = Search;
-//  kdDebug() << m_url.prettyURL() << endl;
+//  myDebug() << m_url.prettyURL() << endl;
   m_job = KIO::get(m_url, false, false);
   connect(m_job, SIGNAL(data(KIO::Job*, const QByteArray&)),
           SLOT(slotData(KIO::Job*, const QByteArray&)));
@@ -140,10 +141,10 @@ void EntrezFetcher::stop() {
     m_job->kill();
     m_job = 0;
   }
-  emit signalDone(this);
   m_data.truncate(0);
   m_started = false;
   m_step = Begin;
+  emit signalDone(this);
 }
 
 void EntrezFetcher::slotData(KIO::Job*, const QByteArray& data_) {
@@ -162,7 +163,7 @@ void EntrezFetcher::slotComplete(KIO::Job* job_) {
   }
 
   if(m_data.isEmpty()) {
-    kdDebug() << "EntrezFetcher::slotComplete() - no data" << endl;
+    myDebug() << "EntrezFetcher::slotComplete() - no data" << endl;
     stop();
     return;
   }
@@ -188,7 +189,7 @@ void EntrezFetcher::slotComplete(KIO::Job* job_) {
     case Begin:
     case Fetch:
     default:
-      kdDebug() << "EntrezFetcher::slotComplete() - wrong step = " << m_step << endl;
+      myDebug() << "EntrezFetcher::slotComplete() - wrong step = " << m_step << endl;
       break;
   }
 }
@@ -239,7 +240,7 @@ void EntrezFetcher::searchResults() {
 
   m_data.truncate(0);
   m_step = Summary;
-//  kdDebug() << m_url.prettyURL() << endl;
+//  myDebug() << m_url.prettyURL() << endl;
   m_job = KIO::get(m_url, false, false);
   connect(m_job, SIGNAL(data(KIO::Job*, const QByteArray&)),
           SLOT(slotData(KIO::Job*, const QByteArray&)));
@@ -263,7 +264,7 @@ void EntrezFetcher::summaryResults() {
     }
     QDomNodeList nodes = e.elementsByTagName(QString::fromLatin1("Id"));
     if(nodes.count() == 0) {
-      kdDebug() << "EntrezFetcher::summaryResults() - no Id elements" << endl;
+      myDebug() << "EntrezFetcher::summaryResults() - no Id elements" << endl;
       continue;
     }
     int id = nodes.item(0).toElement().text().toInt();
@@ -295,11 +296,11 @@ void EntrezFetcher::summaryResults() {
   stop(); // done searching
 }
 
-Tellico::Data::Entry* EntrezFetcher::fetchEntry(uint uid_) {
+Tellico::Data::EntryPtr EntrezFetcher::fetchEntry(uint uid_) {
   // if we already grabbed this one, then just pull it out of the dict
-  const Data::Entry* entry = m_entries[uid_];
+  Data::EntryPtr entry = m_entries[uid_];
   if(entry) {
-    return new Data::Entry(*entry, Data::Document::self()->collection());
+    return entry;
   }
 
   if(!m_matches.contains(uid_)) {
@@ -310,6 +311,7 @@ Tellico::Data::Entry* EntrezFetcher::fetchEntry(uint uid_) {
     initXSLTHandler();
     if(!s_xsltHandler) { // probably an error somewhere in the stylesheet loading
       stop();
+      return 0;
     }
   }
 
@@ -324,20 +326,70 @@ Tellico::Data::Entry* EntrezFetcher::fetchEntry(uint uid_) {
   m_url.addQueryItem(QString::fromLatin1("rettype"),    QString::fromLatin1("abstract"));
   m_url.addQueryItem(QString::fromLatin1("db"),         m_dbname);
   m_url.addQueryItem(QString::fromLatin1("id"),         QString::number(id));
+#endif
+#if 1
+  kdWarning() << "EntrezFetcher::fetchEntry() - turn me off!" << endl;
+  QFile f1(QString::fromLatin1("/tmp/test.xml"));
+  if(f1.open(IO_WriteOnly)) {
+    QTextStream t(&f1);
+    t << FileHandler::readTextFile(m_url);
+  }
+  f1.close();
 #endif
   // now it's sychronous
   QString str = s_xsltHandler->applyStylesheet(FileHandler::readTextFile(m_url));
   Import::TellicoImporter imp(str);
-  Data::Collection* coll = imp.collection();
+  Data::CollPtr coll = imp.collection();
   if(coll->entryCount() == 0) {
-    kdDebug() << "EntrezFetcher::fetchEntry() - no entries in collection" << endl;
+    myDebug() << "EntrezFetcher::fetchEntry() - no entries in collection" << endl;
     return 0;
   } else if(coll->entryCount() > 1) {
-    kdDebug() << "EntrezFetcher::fetchEntry() - collection has multiple entries, taking first one" << endl;
+    myDebug() << "EntrezFetcher::fetchEntry() - collection has multiple entries, taking first one" << endl;
   }
 
-  m_entries.insert(uid_, Data::ConstEntryPtr(coll->entries()[0]));
-  return new Data::Entry(*coll->entries().begin(), Data::Document::self()->collection());
+  Data::EntryPtr e = coll->entries()[0];
+
+  // try to get a link, but only if necessary
+  if(m_fields.contains(QString::fromLatin1("url"))) {
+    KURL link(QString::fromLatin1(ENTREZ_BASE_URL));
+    link.addPath(QString::fromLatin1(ENTREZ_LINK_CGI));
+    link.addQueryItem(QString::fromLatin1("tool"),  QString::fromLatin1("Tellico"));
+    link.addQueryItem(QString::fromLatin1("cmd"),    QString::fromLatin1("llinks"));
+    link.addQueryItem(QString::fromLatin1("db"),     m_dbname);
+    link.addQueryItem(QString::fromLatin1("dbfrom"), m_dbname);
+    link.addQueryItem(QString::fromLatin1("id"),     QString::number(id));
+
+    QDomDocument linkDom = FileHandler::readXMLFile(link, false /* namespace */, true /* quiet */);
+    // need eLinkResult/LinkSet/IdUrlList/IdUrlSet/ObjUrl/Url
+    QDomNode linkNode = linkDom.namedItem(QString::fromLatin1("eLinkResult"))
+                               .namedItem(QString::fromLatin1("LinkSet"))
+                               .namedItem(QString::fromLatin1("IdUrlList"))
+                               .namedItem(QString::fromLatin1("IdUrlSet"))
+                               .namedItem(QString::fromLatin1("ObjUrl"))
+                               .namedItem(QString::fromLatin1("Url"));
+    if(!linkNode.isNull()) {
+      QString u = linkNode.toElement().text();
+      myDebug() << u << endl;
+      if(!u.isEmpty()) {
+        if(!coll->hasField(QString::fromLatin1("url"))) {
+          Data::FieldPtr field = new Data::Field(QString::fromLatin1("url"), i18n("URL"), Data::Field::URL);
+          field->setCategory(i18n("Miscellaneous"));
+          coll->addField(field);
+        }
+        e->setField(QString::fromLatin1("url"), u);
+      }
+    }
+  }
+
+  const StringMap customFields = EntrezFetcher::customFields();
+  for(StringMap::ConstIterator it = customFields.begin(); it != customFields.end(); ++it) {
+    if(!m_fields.contains(it.key())) {
+      coll->removeField(it.key());
+    }
+  }
+
+  m_entries.insert(uid_, e);
+  return e;
 }
 
 void EntrezFetcher::initXSLTHandler() {
@@ -362,14 +414,35 @@ void EntrezFetcher::initXSLTHandler() {
 }
 
 Tellico::Fetch::ConfigWidget* EntrezFetcher::configWidget(QWidget* parent_) const {
-  return new EntrezFetcher::ConfigWidget(parent_);
+  return new EntrezFetcher::ConfigWidget(parent_, this);
 }
 
-EntrezFetcher::ConfigWidget::ConfigWidget(QWidget* parent_)
+EntrezFetcher::ConfigWidget::ConfigWidget(QWidget* parent_, const EntrezFetcher* fetcher_/*=0*/)
     : Fetch::ConfigWidget(parent_) {
-  QVBoxLayout* l = new QVBoxLayout(this);
-  l->addWidget(new QLabel(i18n("This source has no options."), this));
+  QVBoxLayout* l = new QVBoxLayout(optionsWidget());
+  l->addWidget(new QLabel(i18n("This source has no options."), optionsWidget()));
   l->addStretch();
+
+  // now add additional fields widget
+  addFieldsWidget(EntrezFetcher::customFields(), fetcher_ ? fetcher_->m_fields : QStringList());
+}
+
+void EntrezFetcher::ConfigWidget::saveConfig(KConfig* config_) {
+  saveFieldsConfig(config_);
+  slotSetModified(false);
+}
+
+QString EntrezFetcher::ConfigWidget::preferredName() const {
+  return EntrezFetcher::defaultName();
+}
+
+//static
+Tellico::StringMap EntrezFetcher::customFields() {
+  StringMap map;
+  map[QString::fromLatin1("institution")] = i18n("Institution");
+  map[QString::fromLatin1("abstract")]    = i18n("Abstract");
+  map[QString::fromLatin1("url")]         = i18n("URL");
+  return map;
 }
 
 #include "entrezfetcher.moc"
