@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2004 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -16,22 +16,29 @@
 #include "fetch/fetcher.h"
 #include "entryview.h"
 #include "isbnvalidator.h"
+#include "upcvalidator.h"
 #include "tellico_kernel.h"
 #include "filehandler.h"
+#include "collection.h"
+#include "entry.h"
+#include "document.h"
+#include "tellico_debug.h"
+#include "gui/combobox.h"
+#include "gui/listview.h"
+#include "tellico_utils.h"
 
 #include <klocale.h>
-#include <kdebug.h>
-#include <kcombobox.h>
 #include <klineedit.h>
 #include <kpushbutton.h>
 #include <kstatusbar.h>
 #include <khtmlview.h>
 #include <kprogress.h>
 #include <kconfig.h>
-#include <kiconloader.h>
 #include <kdialogbase.h>
-#include <kmessagebox.h>
 #include <kfiledialog.h>
+#include <kiconloader.h>
+#include <kaccelmanager.h>
+#include <ktextedit.h>
 
 #include <qlayout.h>
 #include <qhbox.h>
@@ -54,15 +61,21 @@ namespace {
 
 using Tellico::FetchDialog;
 
-// always add to end
-FetchDialog::SearchResultItem::SearchResultItem(KListView* lv, const Fetch::SearchResult& r)
-    : KListViewItem(lv, lv->lastItem(), QString::null, r.title, r.desc, r.fetcher->source()), m_result(r) {
-}
+class FetchDialog::SearchResultItem : public Tellico::GUI::ListViewItem {
+  friend class FetchDialog;
+  // always add to end
+  SearchResultItem(GUI::ListView* lv, Fetch::SearchResult* r)
+      : GUI::ListViewItem(lv, lv->lastItem()), m_result(r) {
+    setText(1, r->title);
+    setText(2, r->desc);
+    setText(3, r->fetcher->source());
+  }
+  Fetch::SearchResult* m_result;
+};
 
 FetchDialog::FetchDialog(QWidget* parent_, const char* name_)
     : KDialogBase(parent_, name_, false, i18n("Internet Search"), 0),
       m_timer(new QTimer(this)), m_started(false) {
-  m_entries.setAutoDelete(true);
   QWidget* mainWidget = new QWidget(this, "FetchDialog mainWidget");
   setMainWidget(mainWidget);
   QVBoxLayout* topLayout = new QVBoxLayout(mainWidget, 0, KDialog::spacingHint());
@@ -73,19 +86,22 @@ FetchDialog::FetchDialog(QWidget* parent_, const char* name_)
   QHBox* box1 = new QHBox(queryBox, "FetchDialog box1");
   box1->setSpacing(KDialog::spacingHint());
 
-  (void) new QLabel(i18n("Search:"), box1);
+  QLabel* label = new QLabel(i18n("Start the search", "S&earch:"), box1);
 
   m_valueLineEdit = new KLineEdit(box1);
+  label->setBuddy(m_valueLineEdit);
   QWhatsThis::add(m_valueLineEdit, i18n("Enter a search value. An ISBN search must include the full ISBN."));
-  m_keyCombo = new KComboBox(box1);
-  m_keyCombo->insertStringList(Fetch::Manager::self()->keys(QString::null));
-  connect(m_keyCombo, SIGNAL(activated(const QString&)), SLOT(slotKeyChanged(const QString&)));
+  m_keyCombo = new GUI::ComboBox(box1);
+  Fetch::KeyMap map = Fetch::Manager::self()->keyMap();
+  for(Fetch::KeyMap::ConstIterator it = map.begin(); it != map.end(); ++it) {
+    m_keyCombo->insertItem(it.data(), it.key());
+  }
+  connect(m_keyCombo, SIGNAL(activated(int)), SLOT(slotKeyChanged(int)));
   QWhatsThis::add(m_keyCombo, i18n("Choose the type of search"));
 
   m_searchButton = new KPushButton(box1);
   m_searchButton->setGuiItem(KGuiItem(i18n(FETCH_STRING_STOP),
-                             KGlobal::iconLoader()->loadIcon(QString::fromLatin1("cancel"),
-                                                             KIcon::Small)));
+                                      SmallIconSet(QString::fromLatin1("cancel"))));
   connect(m_searchButton, SIGNAL(clicked()), SLOT(slotSearchClicked()));
   QWhatsThis::add(m_searchButton, i18n("Click to start or stop the search"));
 
@@ -93,20 +109,22 @@ FetchDialog::FetchDialog(QWidget* parent_, const char* name_)
   // I don't want it resizing, so figure out the maximum size and set that
   m_searchButton->polish();
   int maxWidth = m_searchButton->sizeHint().width();
+  int maxHeight = m_searchButton->sizeHint().height();
   m_searchButton->setGuiItem(KGuiItem(i18n(FETCH_STRING_SEARCH),
-                             KGlobal::iconLoader()->loadIcon(QString::fromLatin1("find"),
-                                                             KIcon::Small)));
+                                      SmallIconSet(QString::fromLatin1("find"))));
   maxWidth = KMAX(maxWidth, m_searchButton->sizeHint().width());
+  maxHeight = KMAX(maxHeight, m_searchButton->sizeHint().height());
   m_searchButton->setMinimumWidth(maxWidth);
+  m_searchButton->setMinimumHeight(maxHeight);
 
   QHBox* box2 = new QHBox(queryBox);
   box2->setSpacing(KDialog::spacingHint());
 
-  m_multipleISBN = new QCheckBox(i18n("Multiple ISBN search"), box2);
-  QWhatsThis::add(m_multipleISBN, i18n("Check this box to search for multiple ISBN values."));
+  m_multipleISBN = new QCheckBox(i18n("&Multiple ISBN/UPC search"), box2);
+  QWhatsThis::add(m_multipleISBN, i18n("Check this box to search for multiple ISBN or UPC values."));
   connect(m_multipleISBN, SIGNAL(toggled(bool)), SLOT(slotMultipleISBN(bool)));
 
-  m_editISBN = new KPushButton(i18n("Edit ISBN list..."), box2);
+  m_editISBN = new KPushButton(KGuiItem(i18n("Edit List..."), QString::fromLatin1("text_block")), box2);
   m_editISBN->setEnabled(false);
   QWhatsThis::add(m_editISBN, i18n("Click to open a text edit box for entering or editing multiple ISBN values."));
   connect(m_editISBN, SIGNAL(clicked()), SLOT(slotEditMultipleISBN()));
@@ -114,16 +132,20 @@ FetchDialog::FetchDialog(QWidget* parent_, const char* name_)
   // add for spacing
   box2->setStretchFactor(new QWidget(box2), 1);
 
-  (void) new QLabel(i18n("Search Source:"), box2);
+  label = new QLabel(i18n("Search s&ource:"), box2);
   m_sourceCombo = new KComboBox(box2);
-  m_sourceCombo->insertStringList(Fetch::Manager::self()->sources());
+  label->setBuddy(m_sourceCombo);
+  const Fetch::TypePairList sources = Fetch::Manager::self()->sources();
+  for(Fetch::TypePairList::ConstIterator it = sources.begin(); it != sources.end(); ++it) {
+    m_sourceCombo->insertItem((*it).index());
+  }
   connect(m_sourceCombo, SIGNAL(activated(const QString&)), SLOT(slotSourceChanged(const QString&)));
   QWhatsThis::add(m_sourceCombo, i18n("Select the database to search"));
 
   QSplitter* split = new QSplitter(QSplitter::Vertical, mainWidget);
   topLayout->addWidget(split);
 
-  m_listView = new KListView(split);
+  m_listView = new GUI::ListView(split);
 //  topLayout->addWidget(m_listView);
 //  topLayout->setStretchFactor(m_listView, 1);
   m_listView->setSorting(10); // greater than number of columns, so not sorting until user clicks column header
@@ -131,18 +153,22 @@ FetchDialog::FetchDialog(QWidget* parent_, const char* name_)
   m_listView->setAllColumnsShowFocus(true);
   m_listView->setSelectionMode(QListView::Extended);
   m_listView->addColumn(QString::null); // will show a check mark when added
+  m_listView->setColumnAlignment(0, Qt::AlignHCenter); // align checkmark in middle
+  m_listView->setColumnWidthMode(0, QListView::Manual);
   m_listView->addColumn(i18n("Title"));
   m_listView->addColumn(i18n("Description"));
   m_listView->addColumn(i18n("Source"));
-  connect(m_listView, SIGNAL(clicked(QListViewItem*)), SLOT(slotShowEntry(QListViewItem*)));
+
+  connect(m_listView, SIGNAL(selectionChanged()), SLOT(slotShowEntry()));
   // double clicking should add the entry
   connect(m_listView, SIGNAL(doubleClicked(QListViewItem*)), SLOT(slotAddEntry()));
   QWhatsThis::add(m_listView, i18n("As results are found, they are added to this list. Selecting one "
                                    "will fetch the complete entry and show it in the view below."));
 
   m_entryView = new EntryView(split, "entry_view");
-//  topLayout->addWidget(m_entryView->view());
-//  topLayout->setStretchFactor(m_entryView->view(), 2);
+  // don't bother creating funky gradient images for compact view
+  m_entryView->setUseGradientImages(false);
+  // set the xslt file AFTER setting the gradient image option
   m_entryView->setXSLTFile(QString::fromLatin1("Compact.xsl"));
   QWhatsThis::add(m_entryView->view(), i18n("An entry may be shown here before adding it to the "
                                             "current collection by selecting it in the list above"));
@@ -151,13 +177,14 @@ FetchDialog::FetchDialog(QWidget* parent_, const char* name_)
   topLayout->addWidget(box3);
   box3->setSpacing(KDialog::spacingHint());
 
-  m_addButton = new KPushButton(i18n("Add Entry"), box3);
+  m_addButton = new KPushButton(i18n("&Add Entry"), box3);
   m_addButton->setEnabled(false);
+  m_addButton->setIconSet(UserIconSet(Kernel::self()->collectionTypeName()));
   connect(m_addButton, SIGNAL(clicked()), SLOT(slotAddEntry()));
   QWhatsThis::add(m_addButton, i18n("Add the selected entry to the current collection"));
 /*  m_viewButton = new KPushButton(i18n("View Entry"), box3);
   connect(m_viewButton, SIGNAL(clicked()), SLOT(slotViewEntry()));*/
-  KPushButton* clearButton = new KPushButton(i18n("Clear"), box3);
+  KPushButton* clearButton = new KPushButton(KStdGuiItem::clear(), box3);
   connect(clearButton, SIGNAL(clicked()), SLOT(slotClearClicked()));
   QWhatsThis::add(clearButton, i18n("Clear all search fields and results"));
 
@@ -179,143 +206,191 @@ FetchDialog::FetchDialog(QWidget* parent_, const char* name_)
 
   connect(m_timer, SIGNAL(timeout()), SLOT(slotMoveProgress()));
 
-  setMinimumWidth(KMAX(minimumWidth(), FETCH_MIN_WIDTH));
-  slotUpdateStatus(i18n("Ready."));
+  setMinimumWidth(QMAX(minimumWidth(), FETCH_MIN_WIDTH));
+  setStatus(i18n("Ready."));
 
   resize(configDialogSize(QString::fromLatin1("Fetch Dialog Options")));
 
-  KConfig* config = kapp->config();
-  KConfigGroupSaver group(config, "Fetch Dialog Options");
-  QValueList<int> splitList = config->readIntListEntry("Splitter Sizes");
+  KConfigGroup config(kapp->config(), "Fetch Dialog Options");
+  QValueList<int> splitList = config.readIntListEntry("Splitter Sizes");
   if(!splitList.empty()) {
     split->setSizes(splitList);
   }
 
-  connect(Fetch::Manager::self(), SIGNAL(signalResultFound(const Tellico::Fetch::SearchResult&)),
-                          SLOT(slotResultFound(const Tellico::Fetch::SearchResult&)));
-  connect(Fetch::Manager::self(), SIGNAL(signalStatus(const QString&)), SLOT(slotUpdateStatus(const QString&)));
-  connect(Fetch::Manager::self(), SIGNAL(signalDone()), SLOT(slotFetchDone()));
+  connect(Fetch::Manager::self(), SIGNAL(signalResultFound(Tellico::Fetch::SearchResult*)),
+                                  SLOT(slotResultFound(Tellico::Fetch::SearchResult*)));
+  connect(Fetch::Manager::self(), SIGNAL(signalStatus(const QString&)),
+                                  SLOT(slotStatus(const QString&)));
+  connect(Fetch::Manager::self(), SIGNAL(signalDone()),
+                                  SLOT(slotFetchDone()));
 
-  // initialie combos
+  // make sure to delete results afterwards
+  m_results.setAutoDelete(true);
+
+  KAcceleratorManager::manage(this);
+  // initialize combos
   QTimer::singleShot(0, this, SLOT(slotInit()));
 }
 
 FetchDialog::~FetchDialog() {
+  // we might have downloaded a lot of images we don't need to keep
+  Data::EntryVec entriesToCheck;
+  for(QMap<int, Data::EntryPtr>::Iterator it = m_entries.begin(); it != m_entries.end(); ++it) {
+    entriesToCheck.append(it.data());
+  }
+  Data::Document::self()->removeImagesNotInCollection(entriesToCheck);
+
   saveDialogSize(QString::fromLatin1("Fetch Dialog Options"));
 
-  KConfig* config = kapp->config();
-  KConfigGroupSaver group(config, "Fetch Dialog Options");
-  config->writeEntry("Splitter Sizes", static_cast<QSplitter*>(m_listView->parentWidget())->sizes());
-  config->writeEntry("Search Key", m_keyCombo->currentText());
-  config->writeEntry("Search Source", m_sourceCombo->currentText());
+  KConfigGroup config(kapp->config(), "Fetch Dialog Options");
+  config.writeEntry("Splitter Sizes", static_cast<QSplitter*>(m_listView->parentWidget())->sizes());
+  config.writeEntry("Search Key", m_keyCombo->currentData().toInt());
+  config.writeEntry("Search Source", m_sourceCombo->currentText());
 }
 
 void FetchDialog::slotSearchClicked() {
   if(m_started) {
+    setStatus(i18n("Cancelling the search..."));
     Fetch::Manager::self()->stop();
     slotFetchDone();
   } else {
+    QString value = m_valueLineEdit->text().simplifyWhiteSpace();
+    if(value != m_oldSearch) {
+      m_listView->clear();
+      m_entryView->clear();
+    }
+    m_resultCount = 0;
+    m_oldSearch = value;
     m_started = true;
-    m_origCount = m_listView->childCount();
     m_searchButton->setGuiItem(KGuiItem(i18n(FETCH_STRING_STOP),
-                               KGlobal::iconLoader()->loadIcon(QString::fromLatin1("cancel"),
-                                                               KIcon::Small)));
+                                        SmallIconSet(QString::fromLatin1("cancel"))));
     startProgress();
-//    kapp->setOverrideCursor(Qt::waitCursor);
-    slotUpdateStatus(i18n("Searching..."));
+    setStatus(i18n("Searching..."));
     kapp->processEvents();
-    bool multiple = m_multipleISBN->isEnabled() && m_multipleISBN->isChecked();
     Fetch::Manager::self()->startSearch(m_sourceCombo->currentText(),
-                                        Fetch::Manager::fetchKey(m_keyCombo->currentText()),
-                                        m_valueLineEdit->text().simplifyWhiteSpace(),
-                                        multiple);
+                                        static_cast<Fetch::FetchKey>(m_keyCombo->currentData().toInt()),
+                                        value);
   }
 }
 
 void FetchDialog::slotClearClicked() {
   slotFetchDone();
-  m_origCount = 0;
   m_listView->clear();
   m_entryView->clear();
   Fetch::Manager::self()->stop();
   m_valueLineEdit->clear();
+  m_valueLineEdit->setFocus();
   m_addButton->setEnabled(false);
   m_isbnList.clear();
-  slotUpdateStatus(i18n("Ready.")); // because slotFetchDone() writes text
+  m_statusMessages.clear();
+  setStatus(i18n("Ready.")); // because slotFetchDone() writes text
 }
 
-void FetchDialog::slotUpdateStatus(const QString& status_) {
-  m_statusBar->changeItem(QChar(' ') + status_, FETCH_STATUS_ID);
+void FetchDialog::slotStatus(const QString& status_) {
+  m_statusMessages.push_back(status_);
+  // if the queue was empty, start the timer
+  if(m_statusMessages.count() == 1) {
+    // wait 2 seconds
+    QTimer::singleShot(2000, this, SLOT(slotUpdateStatus()));
+  }
+}
+
+void FetchDialog::slotUpdateStatus() {
+  if(m_statusMessages.isEmpty()) {
+    return;
+  }
+  setStatus(m_statusMessages.front());
+  m_statusMessages.pop_front();
+  if(!m_statusMessages.isEmpty()) {
+    // wait 2 seconds
+    QTimer::singleShot(2000, this, SLOT(slotUpdateStatus()));
+  }
+}
+
+void FetchDialog::setStatus(const QString& text_) {
+  m_statusBar->changeItem(QChar(' ') + text_, FETCH_STATUS_ID);
 }
 
 void FetchDialog::slotFetchDone() {
 //  kdDebug() << "FetchDialog::slotFetchDone()" << endl;
   m_started = false;
   m_searchButton->setGuiItem(KGuiItem(i18n(FETCH_STRING_SEARCH),
-                             KGlobal::iconLoader()->loadIcon(QString::fromLatin1("find"),
-                                                             KIcon::Small)));
+                                      SmallIconSet(QString::fromLatin1("find"))));
   stopProgress();
-//  kapp->restoreOverrideCursor();
-  if(m_listView->childCount() == 0) {
-    slotUpdateStatus(i18n("The search returned no items."));
+  if(m_resultCount == 0) {
+    slotStatus(i18n("The search returned no items."));
   } else {
-    slotUpdateStatus(i18n("The search returned 1 item.",
-                          "The search returned %n items.",
-                          m_listView->childCount() - m_origCount));
+    slotStatus(i18n("The search returned 1 item.",
+                    "The search returned %n items.",
+                    m_resultCount));
   }
 }
 
-void FetchDialog::slotResultFound(const Fetch::SearchResult& result_) {
-  new SearchResultItem(m_listView, result_);
+void FetchDialog::slotResultFound(Fetch::SearchResult* result_) {
+  m_results.append(result_);
+  (void) new SearchResultItem(m_listView, result_);
+  ++m_resultCount;
   kapp->processEvents();
 }
 
 void FetchDialog::slotAddEntry() {
-  Data::EntryList list;
-#if QT_VERSION >= 0x030200
+  GUI::CursorSaver cs;
+  Data::EntryVec vec;
   for(QListViewItemIterator it(m_listView, QListViewItemIterator::Selected); it.current(); ++it) {
-#else
-  for(QListViewItemIterator it(m_listView); it.current(); ++it) {
-    if(!it.current()->isSelected()) {
-      continue;
-    }
-#endif
     SearchResultItem* item = static_cast<SearchResultItem*>(it.current());
 
-    const Fetch::SearchResult& r = item->m_result;
-    Data::Entry* entry = m_entries[r.uid];
+    Fetch::SearchResult* r = item->m_result;
+    Data::EntryPtr entry = m_entries[r->uid];
     if(!entry) {
-      entry = r.fetcher->fetchEntry(r.uid);
+      setStatus(i18n("Fetching %1...").arg(r->title));
+      startProgress();
+      entry = r->fetchEntry();
+      if(!entry) {
+        continue;
+      }
+      m_entries.insert(r->uid, entry);
+      stopProgress();
+      setStatus(i18n("Ready."));
     }
-    list.append(entry);
-    // remove it from the dict, since all left over get deleted
-    m_entries.setAutoDelete(false);
-    m_entries.remove(r.uid);
-    m_entries.setAutoDelete(true);
-    item->setPixmap(0, KGlobal::iconLoader()->loadIcon(QString::fromLatin1("ok"), KIcon::Small));
+    // add a copy, intentionally allowing multiple copies to be added
+    vec.append(new Data::Entry(*entry));
+    item->setPixmap(0, UserIcon(QString::fromLatin1("checkmark")));
   }
-
-  if(!list.isEmpty()) {
-    emit signalAddEntries(list);
+  if(!vec.isEmpty()) {
+    Kernel::self()->addEntries(vec, true);
   }
 }
 
-void FetchDialog::slotShowEntry(QListViewItem* item_) {
-  if(!item_) {
+void FetchDialog::slotShowEntry() {
+  // just in case
+  m_statusMessages.clear();
+
+  const GUI::ListViewItemList& items = m_listView->selectedItems();
+  if(items.isEmpty()) {
     m_addButton->setEnabled(false);
     return;
   }
 
   m_addButton->setEnabled(true);
-  SearchResultItem* item = static_cast<SearchResultItem*>(item_);
-  const Fetch::SearchResult& r = item->m_result;
-  slotUpdateStatus(i18n("Fetching %1...").arg(r.title));
-  Data::Entry* entry = m_entries[r.uid];
-  if(!entry) {
-    entry = r.fetcher->fetchEntry(r.uid);
-    m_entries.insert(r.uid, entry);
+  if(items.count() > 1) {
+    m_entryView->clear();
+    return;
   }
-  slotUpdateStatus(i18n("Ready."));
+
+  SearchResultItem* item = static_cast<SearchResultItem*>(items.getFirst());
+  Fetch::SearchResult* r = item->m_result;
+  setStatus(i18n("Fetching %1...").arg(r->title));
+  Data::EntryPtr entry = m_entries[r->uid];
+  if(!entry) {
+    GUI::CursorSaver cs;
+    startProgress();
+    entry = r->fetchEntry();
+    if(entry) { // might conceivably be null
+      m_entries.insert(r->uid, entry);
+    }
+    stopProgress();
+  }
+  setStatus(i18n("Ready."));
 
   m_entryView->showEntry(entry);
 }
@@ -337,51 +412,58 @@ void FetchDialog::stopProgress() {
 void FetchDialog::slotInit() {
   if(!Fetch::Manager::self()->canFetch()) {
     m_searchButton->setEnabled(false);
-    KMessageBox::sorry(this, i18n("No Internet sources are available for your current collection type."));
+    Kernel::self()->sorry(i18n("No Internet sources are available for your current collection type."), this);
   }
 
-  KConfig* config = kapp->config();
-  KConfigGroupSaver group(config, "Fetch Dialog Options");
-  QString key = config->readEntry("Search Key");
+  KConfigGroup config(kapp->config(), "Fetch Dialog Options");
+  int key = config.readNumEntry("Search Key", Fetch::FetchFirst);
   // only change key if valid
-  if(!key.isEmpty() && Fetch::Manager::self()->fetchKey(key) != Fetch::FetchFirst) {
-    slotKeyChanged(key);
-    m_keyCombo->setCurrentText(key);
+  if(key > Fetch::FetchFirst) {
+    m_keyCombo->setCurrentData(key);
   }
-  // only change source if a fetcher by that name exists
-  QString source = config->readEntry("Search Source");
-  if(!source.isEmpty() && Fetch::Manager::self()->sources().findIndex(source) > -1) {
-    slotSourceChanged(source);
-    m_sourceCombo->setCurrentText(source);
+  slotKeyChanged(m_keyCombo->currentItem());
+
+  QString source = config.readEntry("Search Source");
+  if(!source.isEmpty()) {
+    m_sourceCombo->setCurrentItem(source);
   }
+  slotSourceChanged(m_sourceCombo->currentText());
 
   m_valueLineEdit->setFocus();
   m_searchButton->setDefault(true);
 }
 
-void FetchDialog::slotKeyChanged(const QString& key_) {
-  if(key_ == i18n("ISBN")) {
+void FetchDialog::slotKeyChanged(int idx_) {
+  int key = m_keyCombo->data(idx_).toInt();
+  if(key == Fetch::ISBN || key == Fetch::UPC) {
     m_multipleISBN->setEnabled(true);
-    if(Kernel::self()->collection()->type() == Data::Collection::Book
-       || Kernel::self()->collection()->type() == Data::Collection::Bibtex) {
+    if(key == Fetch::ISBN) {
       m_valueLineEdit->setValidator(new ISBNValidator(this));
+    } else {
+      UPCValidator* upc = new UPCValidator(this);
+      connect(upc, SIGNAL(signalISBN()), SLOT(slotUPC2ISBN()));
+      m_valueLineEdit->setValidator(upc);
+      // only want to convert to ISBN if ISBN is accepted by the fetcher
+      Fetch::KeyMap map = Fetch::Manager::self()->keyMap(m_sourceCombo->currentText());
+      upc->setCheckISBN(map.contains(Fetch::ISBN));
     }
   } else {
+    m_multipleISBN->setChecked(false);
     m_multipleISBN->setEnabled(false);
+//    slotMultipleISBN(false);
     m_valueLineEdit->setValidator(0);
   }
 }
 
 void FetchDialog::slotSourceChanged(const QString& source_) {
-  QString s = m_keyCombo->currentText();
+  int curr = m_keyCombo->currentData().toInt();
   m_keyCombo->clear();
-  QStringList sources = Fetch::Manager::self()->keys(source_);
-  m_keyCombo->insertStringList(sources);
-  int idx = sources.findIndex(s);
-  if(idx > -1) {
-    m_keyCombo->setCurrentItem(idx);
+  Fetch::KeyMap map = Fetch::Manager::self()->keyMap(source_);
+  for(Fetch::KeyMap::ConstIterator it = map.begin(); it != map.end(); ++it) {
+    m_keyCombo->insertItem(it.data(), it.key());
   }
-  slotKeyChanged(m_keyCombo->currentText());
+  m_keyCombo->setCurrentData(curr);
+  slotKeyChanged(m_keyCombo->currentItem());
 }
 
 void FetchDialog::slotMultipleISBN(bool toggle_) {
@@ -390,21 +472,23 @@ void FetchDialog::slotMultipleISBN(bool toggle_) {
 }
 
 void FetchDialog::slotEditMultipleISBN() {
-  KDialogBase* dlg = new KDialogBase(this, "isbn edit dialog", true, i18n("Edit ISBN Values"),
-                                     KDialogBase::Ok|KDialogBase::Cancel);
-
-  QVBox* box = new QVBox(dlg);
+  KDialogBase dlg(this, "isbn edit dialog", true, i18n("Edit ISBN/UPC Values"),
+                  KDialogBase::Ok|KDialogBase::Cancel);
+  QVBox* box = new QVBox(&dlg);
   box->setSpacing(10);
-  (void) new QLabel(i18n("<qt>Enter the ISBN values, one per line.</qt>"), box);
+  QString s = i18n("<qt>Enter the ISBN or UPC values, one per line.</qt>");
+  (void) new QLabel(s, box);
   m_isbnTextEdit = new KTextEdit(box, "isbn text edit");
   m_isbnTextEdit->setText(m_isbnList.join(QChar('\n')));
-  QWhatsThis::add(m_isbnTextEdit, i18n("<qt>Enter the ISBN values, one per line.</qt>"));
-  KPushButton* fromFileBtn = new KPushButton(KGlobal::iconLoader()->loadIcon(QString::fromLatin1("fileopen"), KIcon::Small), i18n("Load From File"), box);
-  QWhatsThis::add(fromFileBtn, i18n("<qt>Load the ISBN list from a text file.</qt>"));
+  QWhatsThis::add(m_isbnTextEdit, s);
+  KPushButton* fromFileBtn = new KPushButton(SmallIconSet(QString::fromLatin1("fileopen")),
+                                             i18n("&Load From File..."), box);
+  QWhatsThis::add(fromFileBtn, i18n("<qt>Load the list from a text file.</qt>"));
   connect(fromFileBtn, SIGNAL(clicked()), SLOT(slotLoadISBNList()));
-  dlg->setMainWidget(box);
+  dlg.setMainWidget(box);
+  dlg.setMinimumWidth(KMAX(dlg.minimumWidth(), FETCH_MIN_WIDTH*2/3));
 
-  if(dlg->exec() == QDialog::Accepted) {
+  if(dlg.exec() == QDialog::Accepted) {
     m_isbnList = QStringList::split('\n', m_isbnTextEdit->text());
     const QValidator* val = m_valueLineEdit->validator();
     if(val) {
@@ -418,15 +502,15 @@ void FetchDialog::slotEditMultipleISBN() {
       }
     }
     if(m_isbnList.count() > 100) {
-      KMessageBox::sorry(Kernel::self()->widget(), i18n("<qt>An ISBN search can contain a maximum of 100 ISBN values. Only the first "
-                                                        "100 values in your list will be used.</qt>"));
+      Kernel::self()->sorry(i18n("<qt>An ISBN search can contain a maximum of 100 ISBN values. Only the "
+                                 "first 100 values in your list will be used.</qt>"), this);
     }
     while(m_isbnList.count() > 100) {
       m_isbnList.pop_back();
     }
     m_valueLineEdit->setText(m_isbnList.join(QString::fromLatin1("; ")));
   }
-  dlg->delayedDestruct();
+  m_isbnTextEdit = 0; // gets auto-deleted
 }
 
 void FetchDialog::slotLoadISBNList() {
@@ -441,4 +525,12 @@ void FetchDialog::slotLoadISBNList() {
   }
 }
 
+void FetchDialog::slotUPC2ISBN() {
+  int key = m_keyCombo->currentData().toInt();
+  if(key == Fetch::UPC) {
+    m_keyCombo->setCurrentData(Fetch::ISBN);
+    slotKeyChanged(m_keyCombo->currentItem());
+  }
+}
+
 #include "fetchdialog.moc"
