@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -20,12 +20,17 @@ namespace Tellico {
     class Fetcher;
     class SearchResult;
   }
+  namespace GUI {
+    class ComboBox;
+    class ListView;
+  }
 }
 
 class KComboBox;
 class KLineEdit;
 class KPushButton;
 class KStatusBar;
+class KTextEdit;
 class QProgressBar;
 class QTimer;
 class QCheckBox;
@@ -33,10 +38,7 @@ class QCheckBox;
 #include "datavectors.h"
 
 #include <kdialogbase.h>
-#include <klistview.h>
-#include <ktextedit.h>
 
-#include <qvaluestack.h>
 #include <qguardedptr.h>
 
 namespace Tellico {
@@ -58,7 +60,7 @@ private slots:
   void slotSearchClicked();
   void slotClearClicked();
   void slotAddEntry();
-  void slotShowEntry(QListViewItem* item);
+  void slotShowEntry();
   void slotMoveProgress();
 
   void slotStatus(const QString& status);
@@ -66,31 +68,28 @@ private slots:
 
   void slotFetchDone();
   void slotResultFound(Tellico::Fetch::SearchResult* result);
-  void slotKeyChanged(const QString& key);
+  void slotKeyChanged(int);
   void slotSourceChanged(const QString& source);
   void slotMultipleISBN(bool toggle);
   void slotEditMultipleISBN();
   void slotInit();
   void slotLoadISBNList();
+  void slotUPC2ISBN();
 
 private:
   void startProgress();
   void stopProgress();
   void setStatus(const QString& text);
 
-  class SearchResultItem : public KListViewItem {
-    friend class FetchDialog;
-    SearchResultItem(KListView* lv, Fetch::SearchResult* r);
-    Fetch::SearchResult* m_result;
-  };
+  class SearchResultItem;
 
   KComboBox* m_sourceCombo;
-  KComboBox* m_keyCombo;
+  GUI::ComboBox* m_keyCombo;
   KLineEdit* m_valueLineEdit;
   KPushButton* m_searchButton;
   QCheckBox* m_multipleISBN;
   KPushButton* m_editISBN;
-  KListView* m_listView;
+  GUI::ListView* m_listView;
   EntryView* m_entryView;
   KPushButton* m_addButton;
   KStatusBar* m_statusBar;
@@ -99,11 +98,12 @@ private:
   QGuardedPtr<KTextEdit> m_isbnTextEdit;
 
   bool m_started;
+  int m_resultCount;
   QString m_oldSearch;
   QStringList m_isbnList;
   QStringList m_statusMessages;
   QMap<int, Data::EntryPtr> m_entries;
-  QPtrList<Tellico::Fetch::SearchResult> m_results;
+  QPtrList<Fetch::SearchResult> m_results;
 };
 
 } //end namespace
