@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2001-2005 by Robby Stephenson
+    copyright            : (C) 2001-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -139,9 +139,13 @@ public:
    * Destructor
    */
   ~Field();
-  // default copy constructor is ok, right?
-  Field* clone() { return new Field(*this); }
 
+  /**
+   * Returns the id of the field.
+   *
+   * @return The id
+   */
+  long id() const { return m_id; }
   /**
    * Returns the name of the field.
    *
@@ -246,6 +250,18 @@ public:
    * @param desc The field description
    */
   void setDescription(const QString& desc) { m_desc = desc; }
+  /**
+   * Returns the default value for the field.
+   *
+   * @return The field default value
+   */
+  const QString& defaultValue() const;
+  /**
+   * Sets the default value of the field.
+   *
+   * @param value The field default value
+   */
+  void setDefaultValue(const QString& value);
   /**
    * Some attributes are always a category by themselves.
    *
@@ -311,84 +327,6 @@ public:
    * @param date The string to be formatted
    */
   static QString formatDate(const QString& date);
-  /**
-   * Returns the default article list.
-   *
-   * @return The article list
-   */
-  static QStringList defaultArticleList();
-  /**
-   * Returns the current list of articles.
-   *
-   * @return The article list
-   */
-  static const QStringList& articleList() { return s_articles; }
-  /**
-   * Set the words used as articles.
-   *
-   * @param list The list if articles
-   */
-  static void setArticleList(const QStringList& list);
-  /**
-   * Returns the default suffix list.
-   *
-   * @return The suffix list
-   */
-  static QStringList defaultSuffixList();
-  /**
-   * Returns the list of suffixes used in personal names.
-   *
-   * @return The suffix list
-   */
-  static const QStringList& suffixList() { return s_suffixes; }
-  /**
-   * Set the words used as suffixes in personal names.
-   *
-   * @param list The list of suffixes
-   */
-  static void setSuffixList(const QStringList& list) { s_suffixes = list; }
-  /**
-   * Returns the default surname prefix list.
-   *
-   * @return The prefix list
-   */
-  static QStringList defaultSurnamePrefixList();
-  /**
-   * Returns the list of surname prefixes used in personal names.
-   *
-   * @return The prefix list
-   */
-  static const QStringList& surnamePrefixList() { return s_surnamePrefixes; }
-  /**
-   * Set the words used as surname prefixes in personal names.
-   *
-   * @param list The list of prefixes
-   */
-  static void setSurnamePrefixList(const QStringList& list) { s_surnamePrefixes = list; }
-  /**
-   * Returns true if the capitalization of titles and authors is set to be consistent.
-   *
-   * @return If capitalization is automatically done
-   */
-  static bool autoCapitalize() { return s_autoCapitalize; }
-  /**
-   * Set whether the capitalization of titles and authors is automatic.
-   *
-   * @param autoCapitalize Whether to capitalize or not
-   */
-  static void setAutoCapitalize(bool autoCapitalize) { s_autoCapitalize = autoCapitalize; }
-  /**
-   * Returns true if the titles and authors should be formatted.
-   *
-   * @return If formatting is done
-   */
-  static bool autoFormat() { return s_autoFormat; }
-  /**
-   * Set whether the formatting of titles and authors is automatic.
-   *
-   * @param autoFormat Whether to format or not
-   */
-  static void setAutoFormat(bool autoFormat) { s_autoFormat = autoFormat; }
   /**
    * Helper method to fix capitalization.
    *
@@ -417,9 +355,19 @@ public:
   static const QRegExp& delimiter() { return s_delimiter; }
   /**
    * reset if the field is a rating field used for syntax version 7 and earlier */
-  static void convertOldRating(Data::Field* field);
+  static void convertOldRating(Data::FieldPtr field);
+  static void stripArticles(QString& value);
+  static void articlesUpdated();
+
 
 private:
+  /*
+   * Gets the preferred ID of the collection. Currently, it just gets incremented as
+   * new collections are created.
+   */
+  static long getID();
+
+  long m_id;
   QString m_name;
   QString m_title;
   QString m_category;
@@ -430,14 +378,8 @@ private:
   FormatFlag m_formatFlag;
   StringMap m_properties;
 
-  static QStringList s_articles;
   // need to remember articles with apostrophes for capitalization
   static QStringList s_articlesApos;
-  static QStringList s_suffixes;
-  static QStringList s_surnamePrefixes;
-  static QStringList s_noCapitalize;
-  static bool s_autoCapitalize;
-  static bool s_autoFormat;
   static QRegExp s_delimiter;
 };
 
