@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2001-2005 by Robby Stephenson
+    copyright            : (C) 2001-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -19,16 +19,10 @@
 
 #include <qdict.h>
 #include <qpixmap.h>
-#include <qguardedptr.h>
-
-class KPopupMenu;
 
 namespace Tellico {
   namespace Data {
-    class Collection;
-    class Entry;
     class EntryGroup;
-    class Field;
   }
   class Filter;
   class EntryGroupItem;
@@ -66,7 +60,6 @@ public:
   /**
    * Sets the name of the field by which the entries are grouped
    *
-   * @param coll A pointer to the collection being grouped
    * @param groupFieldName The field name
    */
   void setGroupField(const QString& groupFieldName);
@@ -77,19 +70,19 @@ public:
    *
    * @param coll A pointer to the collection being added
    */
-  void addCollection(Data::Collection* coll);
+  void addCollection(Data::CollPtr coll);
   /**
    * Removes a root collection item, and all of its children.
    *
    * @param coll A pointer to the collection
    */
-  void removeCollection(Data::Collection* coll);
+  void removeCollection(Data::CollPtr coll);
   /**
    * Selects the first item which refers to a certain entry.
    *
    * @param entry A pointer to the entry
    */
-  void setEntrySelected(Data::Entry* entry);
+  void setEntrySelected(Data::EntryPtr entry);
   /**
    * Refresh all the items for the collection.
    *
@@ -97,7 +90,7 @@ public:
    */
   void populateCollection();
 
-  virtual void modifyField(Data::Collection* coll, Data::Field* oldField, Data::Field* newField);
+  void modifyField(Data::CollPtr coll, Data::FieldPtr oldField, Data::FieldPtr newField);
 
 public slots:
   /**
@@ -110,7 +103,7 @@ public slots:
    * @param coll A pointer to the collection of the gorup
    * @param group A pointer to the modified group
    */
-  void slotModifyGroup(Tellico::Data::Collection* coll, Tellico::Data::EntryGroup* group);
+  void slotModifyGroup(Tellico::Data::CollPtr coll, Tellico::Data::EntryGroup* group);
   /**
    * Expands all items at a certain depth. If depth is -1, the current selected item
    * is expanded. If depth is equal to either 0 or 1, then all items at that depth
@@ -180,23 +173,20 @@ signals:
    *
    * @param filter A pointer to the filter
    */
-  void signalUpdateFilter(Tellico::Filter* filter);
+  void signalUpdateFilter(Tellico::FilterPtr filter);
 
 private:
   friend class GroupIterator;
 
   virtual void setSorting(int column, bool ascending = true);
   QString groupTitle();
-  void updateHeader();
+  void updateHeader(Data::FieldPtr field=0);
 
   bool m_notSortedYet;
-  QGuardedPtr<Data::Collection> m_coll;
+  Data::CollPtr m_coll;
   QDict<EntryGroupItem> m_groupDict;
   QString m_groupBy;
 
-  KPopupMenu* m_groupMenu;
-  KPopupMenu* m_entryMenu;
-
   QPixmap m_groupOpenPixmap;
   QPixmap m_groupClosedPixmap;
 };
