@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -19,15 +19,23 @@
 // amarok is licensed under the GPL
 
 #include <kdebug.h>
-#include <ctime>       //std::clock_t
+// std::clock_t
+#include <ctime>
+
+// some logging
+#ifndef NDEBUG
+//#define TELLICO_LOG
+#endif
 
 #ifndef NDEBUG
 #define TELLICO_DEBUG
 #endif
 
 namespace Debug {
+  typedef kndbgstream NoDebugStream;
 #ifndef TELLICO_DEBUG
   typedef kndbgstream DebugStream;
+  static inline DebugStream log()     { return DebugStream(); }
   static inline DebugStream debug()   { return DebugStream(); }
   static inline DebugStream warning() { return DebugStream(); }
   static inline DebugStream error()   { return DebugStream(); }
@@ -41,14 +49,21 @@ namespace Debug {
   #endif
 
 //from kdebug.h
+/*
   enum DebugLevels {
     KDEBUG_INFO  = 0,
     KDEBUG_WARN  = 1,
     KDEBUG_ERROR = 2,
     KDEBUG_FATAL = 3
   };
+*/
 
   typedef kdbgstream DebugStream;
+#ifdef TELLICO_LOG
+  static inline DebugStream log()   { return kdDebug(); }
+#else
+  static inline kndbgstream log()   { return NoDebugStream(); }
+#endif
   static inline DebugStream debug()   { return kdDebug()   << FUNC_PREFIX; }
   static inline DebugStream warning() { return kdWarning() << FUNC_PREFIX << "[WARNING!] "; }
   static inline DebugStream error()   { return kdError()   << FUNC_PREFIX << "[ERROR!] "; }
@@ -57,8 +72,6 @@ namespace Debug {
   #undef FUNC_PREFIX
 #endif
 
-  typedef kndbgstream NoDebugStream;
-
 class Block {
 
 public:
@@ -78,6 +91,10 @@ private :
 };
 
 #define myDebug() Debug::debug()
+#define myWarning() Debug::warning()
+
+#define myLog() Debug::log()
+#define LOG_FUNC Debug::log() << k_funcinfo << endl;
 
 /// Standard function announcer
 #define DEBUG_FUNC_INFO myDebug() << k_funcinfo << endl;
