@@ -26,12 +26,24 @@ int main(int, char**) {
   // check sum value
   check("0-446-60098", "0-446-60098-9");
 
-  // check EAN stripping
-  check("9780672323416", "0-672-32341-9");
+  // check EAN-13
+  check("9780940016750", "978-0-940016-75-0");
+  check("978-0940016750", "978-0-940016-75-0");
+  check("978-0-940016-75-0", "978-0-940016-75-0");
+  check("978286274486", "978-2-86274-486-5");
+  check("9788086119130", "978-80-8611913-7");
+  check("9788086119137", "978-80-8611913-7");
+  check("97880-8611-9-13-0", "978-80-8611-913-7");
+  check("97880-8611-9-13-7", "978-80-8611-913-7");
+
+  // don't add checksum for EAN that start with 978 or 979 and are less than 13 in length
+  check("978059600",   "978-059600");
+  check("978-0596000", "978-059600-0");
 
   // normal english-language hyphenation
-  check("0446600989", "0-446-60098-9");
+  check("0-596-00053", "0-596-00053-7");
   check("044660098", "0-446-60098-9");
+  check("0446600989", "0-446-60098-9");
 
   // check french hyphenation
   check("2862744867", "2-86274-486-7");
