@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -16,16 +16,18 @@
 #include "../collections/bibtexcollection.h"
 #include "../entry.h"
 #include "../latin1literal.h"
+#include "../progressmanager.h"
+#include "../tellico_debug.h"
 
-#include <kdebug.h>
-#include <klocale.h>
+#include <kglobal.h> // for KMAX
+#include <kapplication.h>
 
 #include <qptrlist.h>
 #include <qregexp.h>
 
 using Tellico::Import::BibtexImporter;
 
-BibtexImporter::BibtexImporter(const KURL& url_) : Tellico::Import::TextImporter(url_), m_coll(0) {
+BibtexImporter::BibtexImporter(const KURL& url_) : Tellico::Import::TextImporter(url_), m_coll(0), m_cancelled(false) {
   bt_initialize();
 }
 
@@ -37,31 +39,44 @@ bool BibtexImporter::canImport(int type) const {
   return type == Data::Collection::Bibtex;
 }
 
-Tellico::Data::Collection* BibtexImporter::collection() {
+Tellico::Data::CollPtr BibtexImporter::collection() {
   if(m_coll) {
     return m_coll;
   }
 
-  ASTList list = parseText(text());
-  if(list.isEmpty()) {
+  ProgressItem& item = ProgressManager::self()->newProgressItem(this, progressLabel(), true);
+  item.setTotalSteps(100);
+  connect(&item, SIGNAL(signalCancelled(ProgressItem*)), SLOT(slotCancel()));
+  ProgressItem::Done done(this);
+
+  parseText(text()); // populates m_nodes
+  if(m_cancelled) {
+    return 0;
+  }
+
+  if(m_nodes.isEmpty()) {
     setStatusMessage(i18n("No valid bibtex entries were found in file - %1").arg(url().fileName()));
     return 0;
   } else {
-//    kdDebug() << "BibtexImporter::collection() - found " << list.count() << " entries" << endl;
+//    kdDebug() << "BibtexImporter::collection() - found " << m_nodes.count() << " entries" << endl;
   }
 
   m_coll = new Data::BibtexCollection(true);
-  m_coll->blockSignals(true);
+  Data::BibtexCollection* c = static_cast<Data::BibtexCollection*>(m_coll.data());
 
   QString text;
-  int j = 0;
-  unsigned count = list.count();
-  for(ASTListIterator it(list); it.current(); ++it, ++j) {
+  const uint count = m_nodes.count();
+  const uint stepSize = KMAX(s_stepSize, count/100);
+
+  item.setTotalSteps(count);
+
+  uint j = 0;
+  for(ASTListIterator it(m_nodes); !m_cancelled && it.current(); ++it, ++j) {
     // if we're parsing a macro string, comment or preamble, skip it for now
     if(bt_entry_metatype(it.current()) == BTE_PREAMBLE) {
       char* preamble = bt_get_text(it.current());
       if(preamble) {
-        m_coll->setPreamble(QString::fromLatin1(preamble));
+        c->setPreamble(QString::fromUtf8(preamble));
       }
       continue;
     }
@@ -70,7 +85,8 @@ Tellico::Data::Collection* BibtexImporter::collection() {
       char* macro;
       (void) bt_next_field(it.current(), 0, &macro);
       // FIXME: replace macros within macro definitions!
-      m_coll->addMacro(QString::fromLatin1(macro), QString::fromLatin1(bt_macro_text(macro, 0, 0)));
+      // lookup lowercase macro in map
+      c->addMacro(m_macros[QString::fromUtf8(macro)], QString::fromUtf8(bt_macro_text(macro, 0, 0)));
       continue;
     }
 
@@ -78,15 +94,15 @@ Tellico::Data::Collection* BibtexImporter::collection() {
       continue;
     }
 
-    // now we're parsing a regular entrry
-    Data::Entry* entry = new Data::Entry(m_coll);
+    // now we're parsing a regular entry
+    Data::EntryPtr entry = new Data::Entry(m_coll);
 
-    text = QString::fromLatin1(bt_entry_type(it.current()));
+    text = QString::fromUtf8(bt_entry_type(it.current()));
 //    kdDebug() << "entry type: " << text << endl;
     // text is automatically put into lower-case by btparse
     BibtexHandler::setFieldValue(entry, QString::fromLatin1("entry-type"), text);
 
-    text = QString::fromLatin1(bt_entry_key(it.current()));
+    text = QString::fromUtf8(bt_entry_key(it.current()));
 //    kdDebug() << "entry key: " << text << endl;
     BibtexHandler::setFieldValue(entry, QString::fromLatin1("key"), text);
 
@@ -108,7 +124,7 @@ Tellico::Data::Collection* BibtexImporter::collection() {
             end_macro = false;
             break;
           case BTAST_MACRO:
-            text += QString::fromLatin1(svalue) + QString::fromLatin1("#");
+            text += QString::fromUtf8(svalue) + QString::fromLatin1("#");
             end_macro = true;
             break;
           default:
@@ -119,7 +135,7 @@ Tellico::Data::Collection* BibtexImporter::collection() {
         // remove last character '#'
         text.truncate(text.length() - 1);
       }
-      QString fieldName = QString::fromLatin1(name);
+      QString fieldName = QString::fromUtf8(name);
       if(fieldName == Latin1Literal("author") || fieldName == Latin1Literal("editor")) {
         text.replace(QRegExp(QString::fromLatin1("\\sand\\s")), QString::fromLatin1("; "));
       }
@@ -128,21 +144,29 @@ Tellico::Data::Collection* BibtexImporter::collection() {
 
     m_coll->addEntry(entry);
 
-    if(j%s_stepSize == 0) {
-      emit signalFractionDone(static_cast<float>(j)/static_cast<float>(count));
+    if(j%stepSize == 0) {
+      ProgressManager::self()->setProgress(this, j);
+      kapp->processEvents();
     }
   }
 
+  if(m_cancelled) {
+    m_coll = 0;
+  }
+
   // clean-up
-  for(ASTListIterator it(list); it.current(); ++it) {
+  for(ASTListIterator it(m_nodes); it.current(); ++it) {
     bt_free_ast(it.current());
   }
 
-  m_coll->blockSignals(false);
   return m_coll;
 }
 
-Tellico::Import::BibtexImporter::ASTList BibtexImporter::parseText(const QString& text) const {
+void BibtexImporter::parseText(const QString& text) {
+  if(!m_nodes.isEmpty()) {
+    kdWarning() << "BibtexImporter::parseText() - already parsed the text!" << endl;
+  }
+
   ushort bt_options = 0; // ushort is defined in btparse.h
   boolean ok; // boolean is defined in btparse.h as an int
 
@@ -151,28 +175,33 @@ Tellico::Import::BibtexImporter::ASTList BibtexImporter::parseText(const QString
   bt_set_stringopts(BTE_MACRODEF, 0);
 //  bt_set_stringopts(BTE_PREAMBLE, BTO_CONVERT | BTO_EXPAND);
 
-  ASTList list;
   QString entry;
   QRegExp rx(QString::fromLatin1("[{}]"));
+  QRegExp macroName(QString::fromLatin1("@string\\s*\\{\\s*(.*)="), false /*case sensitive*/);
+  macroName.setMinimal(true);
 
   int brace = 0;
   int startpos = 0;
   int pos = text.find(rx, 0);
-  while(pos > 0) {
+  while(pos > 0 && !m_cancelled) {
     if(text[pos] == '{') {
       ++brace;
     } else if(text[pos] == '}' && brace > 0) {
       --brace;
     }
     if(brace == 0) {
-      entry = text.mid(startpos, pos-startpos+1);
-      //kdDebug() << entry.stripWhiteSpace() << endl;
-      // Assume bibtex file is always encoded as local8Bit
-      AST* node = bt_parse_entry_s(const_cast<char*>(entry.local8Bit().data()),
+      entry = text.mid(startpos, pos-startpos+1).stripWhiteSpace();
+      // All the downstream text processing on the AST node will assume utf-8
+      AST* node = bt_parse_entry_s(const_cast<char*>(entry.utf8().data()),
                                    const_cast<char*>(url().fileName().local8Bit().data()),
                                    0, bt_options, &ok);
       if(ok && node) {
-        list.append(node);
+        if(bt_entry_metatype(node) == BTE_MACRODEF && macroName.search(entry) > -1) {
+          char* macro;
+          (void) bt_next_field(node, 0, &macro);
+          m_macros.insert(QString::fromUtf8(macro), macroName.cap(1).stripWhiteSpace());
+        }
+        m_nodes.append(node);
       }
       startpos = pos+1;
     }
@@ -180,8 +209,10 @@ Tellico::Import::BibtexImporter::ASTList BibtexImporter::parseText(const QString
   }
   // clean up some structures
   bt_parse_entry_s(0, 0, 1, 0, 0);
+}
 
-  return list;
+void BibtexImporter::slotCancel() {
+  m_cancelled = true;
 }
 
 #include "bibteximporter.moc"
