@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -15,16 +15,21 @@
 #include "translators.h" // needed for ImportAction
 #include "../collectionfieldsdialog.h"
 #include "../document.h"
-#include "../field.h"
-#include "../entry.h"
+#include "../collection.h"
+#include "../progressmanager.h"
+#include "../tellico_debug.h"
+#include "../collectionfactory.h"
+#include "../gui/collectiontypecombo.h"
+#include "../latin1literal.h"
+#include "../stringset.h"
 
-#include <kdebug.h>
-#include <klocale.h>
 #include <klineedit.h>
 #include <kcombobox.h>
 #include <knuminput.h>
 #include <kpushbutton.h>
-#include <kaccelmanager.h>
+#include <kapplication.h>
+#include <kiconloader.h>
+#include <kconfig.h>
 
 #include <qgroupbox.h>
 #include <qlayout.h>
@@ -47,71 +52,106 @@ CSVImporter::CSVImporter(const KURL& url_) : Tellico::Import::TextImporter(url_)
     m_existingCollection(0),
     m_firstRowHeader(false),
     m_delimiter(QString::fromLatin1(",")),
+    m_cancelled(false),
     m_widget(0),
     m_table(0) {
 }
 
-Tellico::Data::Collection* CSVImporter::collection() {
+Tellico::Data::CollPtr CSVImporter::collection() {
   // don't just check if m_coll is non-null since the collection can be created elsewhere
   if(m_coll && m_coll->entryCount() > 0) {
     return m_coll;
   }
 
   if(!m_coll) {
-    // iterate over the collection names until it matches the text of the combo box
-    for(CollectionNameMap::Iterator it = m_nameMap.begin(); it != m_nameMap.end(); ++it) {
-      if(it.data() == m_comboType->currentText()) {
-        m_coll = CollectionFactory::collection(it.key(), true);
-        break;
-      }
-    }
+    m_coll = CollectionFactory::collection(m_comboColl->currentType(), true);
   }
 
   QString str = text();
   QTextIStream t(&str);
 
+  const QStringList existingNames = m_coll->fieldNames();
+
   QValueVector<int> cols;
   QStringList names;
   for(int col = 0; col < m_table->numCols(); ++col) {
     QString t = m_table->horizontalHeader()->label(col);
-    if(m_coll->fieldByTitle(t)) {
+    if(m_existingCollection && m_existingCollection->fieldByTitle(t)) {
+      // the collection might have the right field, but a different title, say for translations
+      Data::FieldPtr f = m_existingCollection->fieldByTitle(t);
+      if(m_coll->hasField(f->name())) {
+        // might have different values settings
+        m_coll->removeField(f->name(), true /* force */);
+      }
+      m_coll->addField(new Data::Field(*f));
       cols.push_back(col);
-      names << m_coll->fieldNameByTitle(t);
-    } else if(m_existingCollection && m_existingCollection->fieldByTitle(t)) {
-      m_coll->addField(m_existingCollection->fieldByTitle(t)->clone());
+      names << f->name();
+    } else if(m_coll->fieldByTitle(t)) {
       cols.push_back(col);
       names << m_coll->fieldNameByTitle(t);
     }
   }
 
+  // we don't want to add fields from the default collection
+  // that are not in the imported data
+  for(QStringList::ConstIterator it = existingNames.begin(); it != existingNames.end(); ++it) {
+    if(names.findIndex(*it) == -1) {
+      m_coll->removeField(*it);
+    }
+  }
+
   // if the first row are headers, skip it
   if(m_firstRowHeader) {
     t.readLine();
   }
 
-  int numLines = str.contains(QChar('\n'));
-  int j = 0;
-  for(QString line = t.readLine(); !line.isNull(); line = t.readLine(), ++j) {
+  const uint numLines = str.contains('\n');
+  const uint stepSize = QMAX(s_stepSize, numLines/100);
+
+  ProgressItem& item = ProgressManager::self()->newProgressItem(this, progressLabel(), true);
+  item.setTotalSteps(numLines);
+  connect(&item, SIGNAL(signalCancelled(ProgressItem*)), SLOT(slotCancel()));
+  ProgressItem::Done done(this);
+
+  uint j = 0;
+  for(QString line = t.readLine(); !m_cancelled && !line.isNull(); line = t.readLine(), ++j) {
     bool empty = true;
-    Data::Entry* entry = new Data::Entry(m_coll);
+    Data::EntryPtr entry = new Data::Entry(m_coll);
     QStringList values = splitLine(line);
-    for(unsigned i = 0; i < cols.size(); ++i) {
-      entry->setField(names[i], values[cols[i]].simplifyWhiteSpace());
-      if(empty && !entry->field(names[i]).isEmpty()) {
+    for(uint i = 0; i < names.size(); ++i) {
+      QString value = values[cols[i]].simplifyWhiteSpace();
+      bool success = entry->setField(names[i], value);
+      // we might need to add a new allowed value
+      // assume that if the user is importing the value, it should be allowed
+      if(!success && m_coll->fieldByName(names[i])->type() == Data::Field::Choice) {
+        Data::FieldPtr f = m_coll->fieldByName(names[i]);
+        StringSet allow;
+        allow.add(f->allowed());
+        allow.add(value);
+        f->setAllowed(allow.toList());
+        m_coll->modifyField(f);
+        success = entry->setField(names[i], value);
+      }
+      if(empty && success) {
         empty = false;
       }
     }
-    if(empty) {
-      delete entry;
-    } else {
+    if(!empty) {
       m_coll->addEntry(entry);
     }
 
-    if(j%s_stepSize == 0) {
-      emit signalFractionDone(static_cast<float>(j)/static_cast<float>(numLines));
+    if(j%stepSize == 0) {
+      ProgressManager::self()->setProgress(this, j);
+      kapp->processEvents();
     }
   }
 
+  {
+    KConfigGroup config(KGlobal::config(), QString::fromLatin1("ImportOptions - CSV"));
+    config.writeEntry("Delimiter", m_delimiter);
+    config.writeEntry("First Row Titles", m_firstRowHeader);
+  }
+
   return m_coll;
 }
 
@@ -129,18 +169,15 @@ QWidget* CSVImporter::widget(QWidget* parent_, const char* name_) {
   QHBox* box = new QHBox(group);
   box->setSpacing(5);
   QLabel* lab = new QLabel(i18n("Collection &type:"), box);
-  m_comboType = new KComboBox(box);
-  lab->setBuddy(m_comboType);
-  QWhatsThis::add(m_comboType, i18n("Select the type of collection being imported."));
-  m_nameMap = CollectionFactory::nameMap();
-  m_comboType->insertStringList(m_nameMap.values());
-  connect(m_comboType, SIGNAL(activated(const QString&)), SLOT(slotTypeChanged(const QString&)));
+  m_comboColl = new GUI::CollectionTypeCombo(box);
+  lab->setBuddy(m_comboColl);
+  QWhatsThis::add(m_comboColl, i18n("Select the type of collection being imported."));
+  connect(m_comboColl, SIGNAL(activated(const QString&)), SLOT(slotTypeChanged(const QString&)));
   // need a spacer
   QWidget* w = new QWidget(box);
   box->setStretchFactor(w, 1);
 
-  m_checkFirstRowHeader = new QCheckBox(i18n("First row contains field titles"), group);
-  m_checkFirstRowHeader->setChecked(m_firstRowHeader);
+  m_checkFirstRowHeader = new QCheckBox(i18n("&First row contains field titles"), group);
   QWhatsThis::add(m_checkFirstRowHeader, i18n("If checked, the first row is used as field titles."));
   connect(m_checkFirstRowHeader, SIGNAL(toggled(bool)), SLOT(slotFirstRowHeader(bool)));
 
@@ -153,45 +190,35 @@ QWidget* CSVImporter::widget(QWidget* parent_, const char* name_) {
   connect(m_delimiterGroup, SIGNAL(clicked(int)), SLOT(slotDelimiter()));
 
   m_radioComma = new QRadioButton(m_delimiterGroup);
-  m_radioComma->setText(i18n("Comma"));
+  m_radioComma->setText(i18n("&Comma"));
   m_radioComma->setChecked(true);
   QWhatsThis::add(m_radioComma, i18n("Use a comma as the delimiter."));
   m_delimiterGroupLayout->addWidget(m_radioComma, 1, 0);
 
   m_radioSemicolon = new QRadioButton( m_delimiterGroup);
-  m_radioSemicolon->setText(i18n("Semicolon"));
+  m_radioSemicolon->setText(i18n("&Semicolon"));
   QWhatsThis::add(m_radioSemicolon, i18n("Use a semi-colon as the delimiter."));
   m_delimiterGroupLayout->addWidget(m_radioSemicolon, 1, 1);
 
   m_radioTab = new QRadioButton(m_delimiterGroup);
-  m_radioTab->setText(i18n("Tab"));
+  m_radioTab->setText(i18n("Ta&b"));
   QWhatsThis::add(m_radioTab, i18n("Use a tab as the delimiter."));
   m_delimiterGroupLayout->addWidget(m_radioTab, 2, 0);
 
   m_radioOther = new QRadioButton(m_delimiterGroup);
-  m_radioOther->setText(i18n("Other:"));
+  m_radioOther->setText(i18n("Ot&her:"));
   QWhatsThis::add(m_radioOther, i18n("Use a custom string as the delimiter."));
   m_delimiterGroupLayout->addWidget(m_radioOther, 2, 1);
 
   m_editOther = new KLineEdit(m_delimiterGroup);
-  m_editOther->setEnabled(m_radioOther->isChecked());
+  m_editOther->setEnabled(false);
+  m_editOther->setFixedWidth(m_widget->fontMetrics().width('X') * 4);
   QWhatsThis::add(m_editOther, i18n("A custom string, such as a colon, may be used as a delimiter."));
   m_delimiterGroupLayout->addWidget(m_editOther, 2, 2);
   connect(m_radioOther, SIGNAL(toggled(bool)),
           m_editOther, SLOT(setEnabled(bool)));
   connect(m_editOther, SIGNAL(textChanged(const QString&)), SLOT(slotDelimiter()));
 
-  if(m_delimiter == ',') {
-    m_radioComma->setChecked(true);
-  } else if(m_delimiter == ';') {
-    m_radioSemicolon->setChecked(true);
-  } else if(m_delimiter == '\t') {
-    m_radioTab->setChecked(true);
-  } else {
-    m_radioOther->setChecked(true);
-    m_editOther->setEnabled(true);
-    m_editOther->setText(m_delimiter);
-  }
   w = new QWidget(hbox2);
   hbox2->setStretchFactor(w, 1);
 
@@ -202,33 +229,58 @@ QWidget* CSVImporter::widget(QWidget* parent_, const char* name_) {
   m_table->verticalHeader()->hide();
   m_table->horizontalHeader()->setClickEnabled(true);
   m_table->setReadOnly(true);
+  m_table->setMinimumHeight(m_widget->fontMetrics().lineSpacing() * 8);
   QWhatsThis::add(m_table, i18n("The table shows up to the first five lines of the CSV file."));
   connect(m_table, SIGNAL(currentChanged(int, int)), SLOT(slotCurrentChanged(int, int)));
   connect(m_table->horizontalHeader(), SIGNAL(clicked(int)), SLOT(slotHeaderClicked(int)));
 
-  QHBox* hbox = new QHBox(group);
-  hbox->setSpacing(5);
+  QWidget* hbox = new QWidget(group);
+  QHBoxLayout* hlay = new QHBoxLayout(hbox, 5);
+  hlay->addStretch(10);
   QWhatsThis::add(hbox, i18n("<qt>Set each column to correspond to a field in the collection by choosing "
-                             "a column, selecting the field, then clicking the <i>Set</i> button.</qt>"));
-  lab = new QLabel(i18n("Column:"), hbox);
+                             "a column, selecting the field, then clicking the <i>Assign Field</i> button.</qt>"));
+  lab = new QLabel(i18n("Co&lumn:"), hbox);
+  hlay->addWidget(lab);
   m_colSpinBox = new KIntSpinBox(hbox);
+  hlay->addWidget(m_colSpinBox);
   m_colSpinBox->setMinValue(1);
   connect(m_colSpinBox, SIGNAL(valueChanged(int)), SLOT(slotSelectColumn(int)));
   lab->setBuddy(m_colSpinBox);
+  hlay->addSpacing(10);
 
-  lab = new QLabel(i18n("Data field in this column:"), hbox);
+  lab = new QLabel(i18n("&Data field in this column:"), hbox);
+  hlay->addWidget(lab);
   m_comboField = new KComboBox(hbox);
+  hlay->addWidget(m_comboField);
   connect(m_comboField, SIGNAL(activated(int)), SLOT(slotFieldChanged(int)));
   lab->setBuddy(m_comboField);
+  hlay->addSpacing(10);
 
-  m_setColumnBtn = new KPushButton(i18n("Assign Field"), hbox);
+  m_setColumnBtn = new KPushButton(i18n("&Assign Field"), hbox);
+  hlay->addWidget(m_setColumnBtn);
+  m_setColumnBtn->setIconSet(SmallIconSet(QString::fromLatin1("apply")));
   connect(m_setColumnBtn, SIGNAL(clicked()), SLOT(slotSetColumnTitle()));
-
-  slotTypeChanged(m_comboType->currentText());
-  fillTable();
+  hlay->addStretch(10);
 
   l->addStretch(1);
-  KAcceleratorManager::manage(m_widget);
+
+  KConfigGroup config(KGlobal::config(), QString::fromLatin1("ImportOptions - CSV"));
+  m_delimiter = config.readEntry("Delimiter", m_delimiter);
+  m_firstRowHeader = config.readBoolEntry("First Row Titles", m_firstRowHeader);
+
+  m_checkFirstRowHeader->setChecked(m_firstRowHeader);
+  if(m_delimiter == Latin1Literal(",")) {
+    m_radioComma->setChecked(true);
+  } else if(m_delimiter == Latin1Literal(";")) {
+    m_radioSemicolon->setChecked(true);
+  } else if(m_delimiter == Latin1Literal("\t")) {
+    m_radioTab->setChecked(true);
+  } else {
+    m_radioOther->setChecked(true);
+    m_editOther->setEnabled(true);
+    m_editOther->setText(m_delimiter);
+  }
+
   return m_widget;
 }
 
@@ -237,14 +289,15 @@ QWidget* CSVImporter::widget(QWidget* parent_, const char* name_) {
 // actual quotes may or may not be doubled
 QStringList CSVImporter::splitLine(const QString& line_) {
   // double quote
-  QString dq = QString(s_quote) + QString(s_quote);
+  QString dq = QString(s_quote) + s_quote;
   // delimiter length
-  unsigned dn = m_delimiter.length();
+  uint dn = m_delimiter.length();
   // list of eventual field values
   QStringList values;
   if(dn == 0) {
     return values;
   }
+
   // temporary string to hold splits, and cumulative string for quoted fields
   QString tmp, str;
   // are we inside a quoted field or not
@@ -306,6 +359,8 @@ QStringList CSVImporter::splitLine(const QString& line_) {
     } else if(tmp.startsWith(s_quote)) {
       str = tmp.mid(1, tmp.length()-2);
     }
+  } else {
+    str += tmp;
   }
 
   str.replace(dq, s_quote);
@@ -322,11 +377,10 @@ void CSVImporter::fillTable() {
   QTextStream t(&str, IO_ReadOnly);
 
   int maxCols = 0;
-  // want first two lines
+  QString line;
   for(int row = 0; row < m_table->numRows(); ++row) {
-    QString line = t.readLine();
-//    kdDebug() << "CSVImporter::fillTable() - line is\n" << line << endl;
-    QStringList values = splitLine(line);
+    line = t.readLine();
+    const QStringList values = splitLine(line);
     if(static_cast<int>(values.count()) > m_table->numCols()) {
       m_table->setNumCols(values.count());
       m_colSpinBox->setMaxValue(values.count());
@@ -344,17 +398,10 @@ void CSVImporter::fillTable() {
   m_table->setNumCols(maxCols);
 }
 
-void CSVImporter::slotTypeChanged(const QString& name_) {
-//  kdDebug() << "CSVImporter::slotTypeChanged() - new type = " << name_ << endl;
+void CSVImporter::slotTypeChanged(const QString&) {
   // iterate over the collection names until it matches the text of the combo box
-  for(CollectionNameMap::Iterator it = m_nameMap.begin(); it != m_nameMap.end(); ++it) {
-    if(it.data() == name_) {
-    // delete the old collection
-      delete m_coll;
-      m_coll = CollectionFactory::collection(it.key(), true);
-      break;
-    }
-  }
+  Data::Collection::Type type = static_cast<Data::Collection::Type>(m_comboColl->currentType());
+  m_coll = CollectionFactory::collection(type, true);
 
   updateHeader(true);
   m_comboField->clear();
@@ -373,12 +420,13 @@ void CSVImporter::slotFirstRowHeader(bool b_) {
 
 void CSVImporter::slotDelimiter() {
   if(m_radioComma->isChecked()) {
-    m_delimiter = QString::fromLatin1(",");
+    m_delimiter = ',';
   } else if(m_radioSemicolon->isChecked()) {
-    m_delimiter = QString::fromLatin1(";");
+    m_delimiter = ';';
   } else if(m_radioTab->isChecked()) {
-    m_delimiter = QString::fromLatin1("\t");
+    m_delimiter = '\t';
   } else {
+    m_editOther->setFocus();
     m_delimiter = m_editOther->text();
   }
   if(!m_delimiter.isEmpty()) {
@@ -437,11 +485,11 @@ void CSVImporter::updateHeader(bool force_) {
     return;
   }
 
-  Data::Collection* c = m_existingCollection ? m_existingCollection : m_coll;
+  Data::CollPtr c = m_existingCollection ? m_existingCollection : m_coll;
   for(int col = 0; col < m_table->numCols(); ++col) {
-    if(m_firstRowHeader && m_firstRowHeader
-       && c->fieldByTitle(m_table->text(0, col)) != 0) {
-      m_table->horizontalHeader()->setLabel(col, m_table->text(0, col));
+    const QString s = m_table->text(0, col);
+    if(m_firstRowHeader && !s.isEmpty() && c && c->fieldByTitle(s) != 0) {
+      m_table->horizontalHeader()->setLabel(col, s);
     } else {
       m_table->horizontalHeader()->setLabel(col, QString::number(col+1));
     }
@@ -454,8 +502,8 @@ void CSVImporter::slotFieldChanged(int idx_) {
     return;
   }
 
-  Data::Collection* c = m_existingCollection ? m_existingCollection : m_coll;
-  unsigned count = c->fieldTitles().count();
+  Data::CollPtr c = m_existingCollection ? m_existingCollection : m_coll;
+  uint count = c->fieldTitles().count();
   CollectionFieldsDialog dlg(c, m_widget);
 //  dlg.setModal(true);
   if(dlg.exec() == QDialog::Accepted) {
@@ -470,8 +518,8 @@ void CSVImporter::slotFieldChanged(int idx_) {
 }
 
 void CSVImporter::slotActionChanged(int action_) {
-  Data::Collection* m_currColl = Data::Document::self()->collection();
-  if(!m_currColl) {
+  Data::CollPtr currColl = Data::Document::self()->collection();
+  if(!currColl) {
     m_existingCollection = 0;
     return;
   }
@@ -479,26 +527,28 @@ void CSVImporter::slotActionChanged(int action_) {
   switch(action_) {
     case Import::Replace:
       {
-        QString currText = m_comboType->currentText();
-        m_comboType->clear();
-        m_comboType->insertStringList(m_nameMap.values());
-        m_comboType->setCurrentText(currText);
+        int currType = m_comboColl->currentType();
+        m_comboColl->reset();
+        m_comboColl->setCurrentType(currType);
         m_existingCollection = 0;
       }
       break;
 
     case Import::Append:
     case Import::Merge:
-      m_comboType->clear();
-      m_comboType->insertItem(m_nameMap[m_currColl->type()]);
-      m_existingCollection = m_currColl;
-      break;
-
-
-    default:
-      break;
+     {
+        m_comboColl->clear();
+        QString name = CollectionFactory::nameMap()[currColl->type()];
+        m_comboColl->insertItem(name, currColl->type());
+        m_existingCollection = currColl;
+     }
+     break;
   }
-  slotTypeChanged(m_comboType->currentText());
+  slotTypeChanged(m_comboColl->currentText());
+}
+
+void CSVImporter::slotCancel() {
+  m_cancelled = true;
 }
 
 #include "csvimporter.moc"
