@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -14,7 +14,6 @@
 #include "htmlexporter.h"
 #include "xslthandler.h"
 #include "tellicoxmlexporter.h"
-#include "tellico_xml.h"
 #include "../document.h"
 #include "../collection.h"
 #include "../filehandler.h"
@@ -22,17 +21,14 @@
 #include "../latin1literal.h"
 #include "../tellico_kernel.h"
 #include "../tellico_utils.h"
+#include "../progressmanager.h"
+#include "../core/tellico_config.h"
 
 #include <kstandarddirs.h>
 #include <kconfig.h>
-#include <kdebug.h>
-#include <kglobalsettings.h>
+#include <kglobal.h>
 #include <kio/netaccess.h>
-#include <kdeversion.h>
 #include <kapplication.h>
-#include <khtml_part.h>
-#include <dom/dom_doc.h>
-#include <dom/html_element.h>
 
 #include <qdom.h>
 #include <qgroupbox.h>
@@ -42,28 +38,36 @@
 #include <qfile.h>
 #include <qhbox.h>
 #include <qlabel.h>
-#include <qapplication.h>
+
+extern "C" {
+#include <libxml/HTMLparser.h>
+#include <libxml/HTMLtree.h>
+}
 
 using Tellico::Export::HTMLExporter;
 
 HTMLExporter::HTMLExporter() : Tellico::Export::Exporter(),
-    m_part(0),
     m_handler(0),
     m_printHeaders(true),
     m_printGrouped(false),
     m_exportEntryFiles(false),
+    m_cancelled(false),
+    m_parseDOM(true),
+    m_checkCreateDir(true),
     m_imageWidth(0),
     m_imageHeight(0),
     m_widget(0),
     m_xsltFile(QString::fromLatin1("tellico2html.xsl")) {
 }
 
-HTMLExporter::HTMLExporter(const Data::Collection* coll_) : Tellico::Export::Exporter(coll_),
-    m_part(0),
+HTMLExporter::HTMLExporter(Data::CollPtr coll_) : Tellico::Export::Exporter(coll_),
     m_handler(0),
     m_printHeaders(true),
     m_printGrouped(false),
     m_exportEntryFiles(false),
+    m_cancelled(false),
+    m_parseDOM(true),
+    m_checkCreateDir(true),
     m_imageWidth(0),
     m_imageHeight(0),
     m_widget(0),
@@ -71,8 +75,6 @@ HTMLExporter::HTMLExporter(const Data::Collection* coll_) : Tellico::Export::Exp
 }
 
 HTMLExporter::~HTMLExporter() {
-  delete m_part;
-  m_part = 0;
   delete m_handler;
   m_handler = 0;
 }
@@ -85,45 +87,79 @@ QString HTMLExporter::fileFilter() const {
   return i18n("*.html|HTML Files (*.html)") + QChar('\n') + i18n("*|All Files");
 }
 
-bool HTMLExporter::exec() {
-  if(url().isEmpty() || !url().isValid()) {
-    return false;
-  }
-
-  // this is expensive, but necessary to easily crawl the DOM and fetch external images
-  if(!m_part) {
-    m_part = new KHTMLPart((QWidget*)0);
-    m_part->setJScriptEnabled(false);
-    m_part->setJavaEnabled(false);
-    m_part->setMetaRefreshEnabled(false);
-    m_part->setPluginsEnabled(false);
-  }
-
-  m_part->begin();
-  m_part->write(text());
-  m_part->end();
-
-  DOM::HTMLElement e = m_part->document().documentElement();
-  // a little paranoia, the XSL transform might fail
-  if(e.isNull() || !e.hasChildNodes()) {
-    return false;
-  }
-  parseNode(e);
-
-  bool success = FileHandler::writeTextURL(url(),
-                                           QString::fromLatin1("<html>")
-                                             + e.innerHTML().string()
-                                             + QString::fromLatin1("</html>"),
-                                           options() & Export::ExportUTF8,
-                                           options() & Export::ExportForce);
-  return success && copyFiles() && (!m_exportEntryFiles || writeEntryFiles());
+void HTMLExporter::reset() {
+  // since the ExportUTF8 option may have changed, need to delete handler
+  delete m_handler;
+  m_handler = 0;
+  m_files.clear();
+  m_links.clear();
+  m_copiedFiles.clear();
 }
 
-QString HTMLExporter::text() {
+bool HTMLExporter::exec() {
+  if(url().isEmpty() || !url().isValid()) {
+    kdWarning() << "HTMLExporter::exec() - trying to export to invalid URL" << endl;
+    return false;
+  }
+
+  // check file exists first
+  // if we're not forcing, ask use
+  bool force = (options() & Export::ExportForce) || FileHandler::queryExists(url());
+  if(!force) {
+    return false;
+  }
+
+  if(!m_parseDOM) {
+    return FileHandler::writeTextURL(url(), text(), options() & Export::ExportUTF8, force);
+  }
+
+  m_cancelled = false;
+  // TODO: maybe need label?
+  if(options() & ExportProgress) {
+    ProgressItem& item = ProgressManager::self()->newProgressItem(this, QString::null, true);
+    item.setTotalSteps(100);
+    connect(&item, SIGNAL(signalCancelled(ProgressItem*)), SLOT(slotCancel()));
+  }
+  // ok if not ExportProgress, no worries
+  ProgressItem::Done done(this);
+
+  htmlDocPtr htmlDoc = htmlParseDoc(reinterpret_cast<xmlChar*>(text().utf8().data()), NULL);
+  xmlNodePtr root = xmlDocGetRootElement(htmlDoc);
+  if(root == 0) {
+    myDebug() << "HTMLExporter::exec() - no root" << endl;
+    return false;
+  }
+  parseDOM(root);
+
+  if(m_cancelled) {
+    return true; // intentionally cancelled
+  }
+  ProgressManager::self()->setProgress(this, 15);
+
+  xmlChar* c;
+  int bytes;
+  htmlDocDumpMemory(htmlDoc, &c, &bytes);
+  QString allText;
+  if(bytes > 0) {
+    allText = QString::fromUtf8(reinterpret_cast<const char*>(c), bytes);
+    xmlFree(c);
+  }
+
+  if(m_cancelled) {
+    return true; // intentionally cancelled
+  }
+  ProgressManager::self()->setProgress(this, 20);
+
+  bool success = FileHandler::writeTextURL(url(), allText, options() & Export::ExportUTF8, force);
+  success &= copyFiles() && (!m_exportEntryFiles || writeEntryFiles());
+  return success;
+}
+
+bool HTMLExporter::loadXSLTFile() {
   QString xsltfile = locate("appdata", m_xsltFile);
   if(xsltfile.isNull()) {
-    kdDebug() << "HTMLExporter::text() - no xslt file for " << m_xsltFile << endl;
-    return QString::null;
+    myDebug() << "HTMLExporter::loadXSLTFile() - no xslt file for " << m_xsltFile << endl;
+    return false;
   }
 
   KURL u;
@@ -133,8 +169,8 @@ QString HTMLExporter::text() {
   // removes the namespace declaration
   QDomDocument dom = FileHandler::readXMLFile(u, false);
   if(dom.isNull()) {
-    kdDebug() << "HTMLExporter::text() - error loading xslt file: " << xsltfile << endl;
-    return QString::null;
+    myDebug() << "HTMLExporter::loadXSLTFile() - error loading xslt file: " << xsltfile << endl;
+    return false;
   }
 
   // notes about utf-8 encoding:
@@ -147,7 +183,50 @@ QString HTMLExporter::text() {
     XSLTHandler::setLocaleEncoding(dom);
   }
 
-  const Data::Collection* coll = collection();
+  delete m_handler;
+  m_handler = new XSLTHandler(dom, QFile::encodeName(xsltfile), true /*translate*/);
+  if(!m_handler->isValid()) {
+    delete m_handler;
+    m_handler = 0;
+    return false;
+  }
+
+  if(m_exportEntryFiles) {
+    // export entries to same place as all the other date files
+    m_handler->addStringParam("entrydir", QFile::encodeName(fileDir().fileName())+ '/');
+    // be sure to link all the entries
+    m_handler->addParam("link-entries", "true()");
+  }
+
+  if(!m_collectionURL.isEmpty()) {
+    QString s = QString::fromLatin1("../") + m_collectionURL.fileName();
+    m_handler->addStringParam("collection-file", s.utf8());
+  }
+
+  // look for a file that gets installed to know the installation directory
+  // if parseDOM, that means we want the locations to be the actual location
+  // otherwise, we assume it'll be relative
+  if(m_parseDOM && m_dataDir.isEmpty()) {
+    m_dataDir = KGlobal::dirs()->findResourceDir("appdata", QString::fromLatin1("pics/tellico.png"));
+  } else if(!m_parseDOM) {
+    m_dataDir.truncate(0);
+  }
+  if(!m_dataDir.isEmpty()) {
+    m_handler->addStringParam("datadir", QFile::encodeName(m_dataDir));
+  }
+
+  setFormattingOptions(collection());
+
+  return m_handler->isValid();
+}
+
+QString HTMLExporter::text() {
+  if((!m_handler || !m_handler->isValid()) && !loadXSLTFile()) {
+    kdWarning() << "HTMLExporter::text() - error loading xslt file: " << m_xsltFile << endl;
+    return QString::null;
+  }
+
+  Data::CollPtr coll = collection();
   if(!coll) {
     myDebug() << "HTMLExporter::text() - no collection pointer!" << endl;
     return QString::null;
@@ -157,35 +236,7 @@ QString HTMLExporter::text() {
     m_printGrouped = false; // can't group if no groups exist
   }
 
-  if(!m_handler) {
-    m_handler = new XSLTHandler(dom, QFile::encodeName(xsltfile));
-    if(!m_handler->isValid()) {
-      delete m_handler;
-      m_handler = 0;
-      kdWarning() << "HTMLExporter::text() - error loading xslt file: " << xsltfile << endl;
-      return QString::null;
-    }
-    setFormattingOptions(coll);
-
-    if(m_exportEntryFiles) {
-      // export entries to same place as all the other date files
-      m_handler->addStringParam("entrydir", QFile::encodeName(fileDir().fileName())+ '/');
-      // be sure to link all the entries
-      m_handler->addParam("link-entries", "true()");
-    }
-
-    if(!m_collectionURL.isEmpty()) {
-      QString s = QString::fromLatin1("../") + m_collectionURL.fileName();
-      m_handler->addStringParam("collection-file", s.utf8());
-    }
-
-    // look for a file that gets installed to know the installation directory
-    QString dataDir = KGlobal::dirs()->findResourceDir("appdata", QString::fromLatin1("pics/tellico.png"));
-    m_handler->addStringParam("datadir", QFile::encodeName(dataDir));
-  }
-
-  GUI::CursorSaver cs(Qt::waitCursor);
-
+  GUI::CursorSaver cs;
   writeImages(coll);
 
   // now grab the XML
@@ -211,15 +262,22 @@ QString HTMLExporter::text() {
   if(f2.open(IO_WriteOnly)) {
     QTextStream t(&f2);
     t << text;
-    t << "\n\n-------------------------------------------------------\n\n";
-    t << Tellico::i18nReplace(text);
+//    t << "\n\n-------------------------------------------------------\n\n";
+//    t << Tellico::i18nReplace(text);
   }
   f2.close();
 #endif
-  return Tellico::i18nReplace(text);
+  // the XSLT file gets translated instead
+//  return Tellico::i18nReplace(text);
+  return text;
 }
 
-void HTMLExporter::setFormattingOptions(const Data::Collection* coll) {
+void HTMLExporter::setFormattingOptions(Data::CollPtr coll) {
+  QString file = Kernel::self()->URL().fileName();
+  if(file != i18n("Untitled")) {
+    m_handler->addStringParam("filename", QFile::encodeName(file));
+  }
+  m_handler->addStringParam("cdate", KGlobal::locale()->formatDate(QDate::currentDate()).utf8());
   m_handler->addParam("show-headers", m_printHeaders ? "true()" : "false()");
   m_handler->addParam("group-entries", m_printGrouped ? "true()" : "false()");
 
@@ -261,7 +319,7 @@ void HTMLExporter::setFormattingOptions(const Data::Collection* coll) {
 
     QString groupFields;
     for(QStringList::ConstIterator it = m_groupBy.begin(); it != m_groupBy.end(); ++it) {
-      Data::Field* f = coll->fieldByName(*it);
+      Data::FieldPtr f = coll->fieldByName(*it);
       if(!f) {
         continue;
       }
@@ -285,12 +343,12 @@ void HTMLExporter::setFormattingOptions(const Data::Collection* coll) {
         groupFields += '|';
       }
     }
-//    kdDebug() << groupFields << endl;
+//    myDebug() << groupFields << endl;
     m_handler->addStringParam("group-fields", groupFields.utf8());
     m_handler->addStringParam("sort-title", sortString.utf8());
   }
 
-  QString pageTitle = QString::fromLatin1("Tellico: ") + coll->title();
+  QString pageTitle = coll->title();
   pageTitle += QChar(' ') + sortString;
   m_handler->addStringParam("page-title", pageTitle.utf8());
 
@@ -306,20 +364,24 @@ void HTMLExporter::setFormattingOptions(const Data::Collection* coll) {
   }
 
   // add system colors to stylesheet
-  const QColorGroup& cg = QApplication::palette().active();
-  m_handler->addStringParam("font",    KGlobalSettings::generalFont().family().utf8());
-  m_handler->addStringParam("bgcolor", cg.base().name().utf8());
-  m_handler->addStringParam("fgcolor", cg.text().name().utf8());
-  m_handler->addStringParam("color1",  cg.highlightedText().name().utf8());
-  m_handler->addStringParam("color2",  cg.highlight().name().utf8());
+  const int type = coll->type();
+  m_handler->addStringParam("font",     Config::templateFont(type).family().latin1());
+  m_handler->addStringParam("fontsize", QCString().setNum(Config::templateFont(type).pointSize()));
+  m_handler->addStringParam("bgcolor",  Config::templateBaseColor(type).name().latin1());
+  m_handler->addStringParam("fgcolor",  Config::templateTextColor(type).name().latin1());
+  m_handler->addStringParam("color1",   Config::templateHighlightedTextColor(type).name().latin1());
+  m_handler->addStringParam("color2",   Config::templateHighlightedBaseColor(type).name().latin1());
+
+  // add locale code to stylesheet (for sorting)
+  m_handler->addStringParam("lang",  KGlobal::locale()->languagesTwoAlpha().first().utf8());
 }
 
-void HTMLExporter::writeImages(const Data::Collection* coll_) {
+void HTMLExporter::writeImages(Data::CollPtr coll_) {
   // keep track of which image fields to write, this is for field names
-  QStringList imageFields;
+  StringSet imageFields;
   for(QStringList::ConstIterator it = m_columns.begin(); it != m_columns.end(); ++it) {
     if(coll_->fieldByTitle(*it)->type() == Data::Field::Image) {
-      imageFields << *it;
+      imageFields.add(*it);
     }
   }
 
@@ -331,33 +393,55 @@ void HTMLExporter::writeImages(const Data::Collection* coll_) {
     // add all image fields to string list
     Data::FieldVec fields = coll_->imageFields();
     for(Data::FieldVec::Iterator fieldIt = fields.begin(); fieldIt != fields.end(); ++fieldIt) {
-      const QString& fieldName = fieldIt->name();
-      if(imageFields.findIndex(fieldName) == -1) { // be sure not to have duplicate values
-        imageFields << fieldName;
-      }
+      imageFields.add(fieldIt->name());
     }
   }
 
   // all of them are going to get written to tmp file
-  m_handler->addStringParam("imgdir", QFile::encodeName(ImageFactory::tempDir()));
+  bool useTemp = url().isEmpty();
+  KURL imgDir;
+  QString imgDirRelative;
+  // really some convoluted logic here
+  // basically, four cases. 1) we're writing to a tmp file, for printing probably
+  // so then write all the images to the tmp directory, 2) we're exporting to HTML, and
+  // this is the main collection file, in which case m_parseDOM is always true;
+  // 3) we're exporting HTML, and this is the first entry file, for which parseDOM is true
+  // and exportEntryFiles is false. Then the image file will get copied in copyFiles() and is
+  // probably an image in the entry template. 4) we're exporting HTML, and this is not the
+  // first entry file, in which case, we want to refer directly to the target dir
+  if(useTemp) { // everything goes in the tmp dir
+    imgDir.setPath(ImageFactory::tempDir());
+    imgDirRelative = imgDir.path();
+  } else if(m_parseDOM) {
+    imgDir = fileDir(); // copy to fileDir
+    imgDirRelative = Data::Document::self()->allImagesOnDisk() ? ImageFactory::dataDir() : ImageFactory::tempDir();
+    createDir();
+  } else {
+    imgDir = fileDir();
+    imgDirRelative = KURL::relativeURL(url(), imgDir);
+    createDir();
+  }
+  m_handler->addStringParam("imgdir", QFile::encodeName(imgDirRelative));
 
-  // call kapp->processEvents(), too
   int count = 0;
   const int processCount = 100; // process after every 100 events
 
+  QStringList fieldsList = imageFields.toList();
   StringSet imageSet; // track which images are written
-  for(QStringList::ConstIterator fieldName = imageFields.begin(); fieldName != imageFields.end(); ++fieldName) {
+  for(QStringList::ConstIterator fieldName = fieldsList.begin(); fieldName != fieldsList.end(); ++fieldName) {
     for(Data::EntryVec::ConstIterator entryIt = entries().begin(); entryIt != entries().end(); ++entryIt) {
-      const QString& id = entryIt->field(*fieldName);
+      QString id = entryIt->field(*fieldName);
       // if no id or is already writen, continue
       if(id.isEmpty() || imageSet.has(id)) {
         continue;
       }
       imageSet.add(id);
       // try writing
-      if(!ImageFactory::writeImage(id, ImageFactory::tempDir(), true)) {
-        kdWarning() << "HTMLExporter::write image() - unable to write image file: "
-                    << ImageFactory::tempDir() << id << endl;
+      bool success = useTemp ? ImageFactory::writeCachedImage(id, ImageFactory::TempDir)
+                             : ImageFactory::writeImage(id, imgDir, true);
+      if(!success) {
+        kdWarning() << "HTMLExporter::writeImages() - unable to write image file: "
+                    << imgDir.path() << id << endl;
       }
 
       if(++count == processCount) {
@@ -398,26 +482,25 @@ QWidget* HTMLExporter::widget(QWidget* parent_, const char* name_/*=0*/) {
 }
 
 void HTMLExporter::readOptions(KConfig* config_) {
-  KConfigGroupSaver group(config_, QString::fromLatin1("ExportOptions - %1").arg(formatString()));
-  m_printHeaders = config_->readBoolEntry("Print Field Headers", m_printHeaders);
-  m_printGrouped = config_->readBoolEntry("Print Grouped", m_printGrouped);
-  m_exportEntryFiles = config_->readBoolEntry("Export Entry Files", m_exportEntryFiles);
+  KConfigGroup exportConfig(config_, QString::fromLatin1("ExportOptions - %1").arg(formatString()));
+  m_printHeaders = exportConfig.readBoolEntry("Print Field Headers", m_printHeaders);
+  m_printGrouped = exportConfig.readBoolEntry("Print Grouped", m_printGrouped);
+  m_exportEntryFiles = exportConfig.readBoolEntry("Export Entry Files", m_exportEntryFiles);
 
   // read current entry export template
-  config_->setGroup(QString::fromLatin1("Options - %1").arg(collection()->entryName()));
-  m_entryXSLTFile = config_->readEntry("Entry Template", QString::fromLatin1("Default"));
+  m_entryXSLTFile = Config::templateName(collection()->type());
   m_entryXSLTFile = locate("appdata", QString::fromLatin1("entry-templates/")
                                       + m_entryXSLTFile + QString::fromLatin1(".xsl"));
 }
 
 void HTMLExporter::saveOptions(KConfig* config_) {
-  KConfigGroupSaver group(config_, QString::fromLatin1("ExportOptions - %1").arg(formatString()));
+  KConfigGroup cfg(config_, QString::fromLatin1("ExportOptions - %1").arg(formatString()));
   m_printHeaders = m_checkPrintHeaders->isChecked();
-  config_->writeEntry("Print Field Headers", m_printHeaders);
+  cfg.writeEntry("Print Field Headers", m_printHeaders);
   m_printGrouped = m_checkPrintGrouped->isChecked();
-  config_->writeEntry("Print Grouped", m_printGrouped);
+  cfg.writeEntry("Print Grouped", m_printGrouped);
   m_exportEntryFiles = m_checkExportEntryFiles->isChecked();
-  config_->writeEntry("Export Entry Files", m_exportEntryFiles);
+  cfg.writeEntry("Export Entry Files", m_exportEntryFiles);
 }
 
 void HTMLExporter::setXSLTFile(const QString& filename_) {
@@ -427,15 +510,13 @@ void HTMLExporter::setXSLTFile(const QString& filename_) {
 
   m_xsltFile = filename_;
   m_xsltFilePath = QString::null;
-  delete m_handler;
-  m_handler = 0;
+  reset();
 }
 
 KURL HTMLExporter::fileDir() const {
   if(url().isEmpty()) {
     return KURL();
   }
-
   KURL fileDir = url();
   // cd to directory of target URL
   fileDir.cd(QString::fromLatin1(".."));
@@ -445,45 +526,14 @@ KURL HTMLExporter::fileDir() const {
 
 QString HTMLExporter::fileDirName() const {
   if(!m_collectionURL.isEmpty()) {
-    return m_collectionURL.fileName().section('.', 0, 0) + QString::fromLatin1("_files/");
+    return QString::fromLatin1("/");
   }
   return url().fileName().section('.', 0, 0) + QString::fromLatin1("_files/");
 }
 
-void HTMLExporter::parseNode(DOM::Node node_) {
-  DOM::Element elem = node_;
-  if(!elem.isNull()) {
-    const DOM::DOMString nodeName = node_.nodeName().upper();
-    // to speed up things, check now for nodename
-    if(nodeName == "IMG" || nodeName == "SCRIPT" || nodeName == "LINK") {
-      DOM::DOMString attrName;
-      DOM::Attr attr;
-      DOM::NamedNodeMap attrs = elem.attributes();
-      const uint lmap = attrs.length();
-      for(uint j = 0; j < lmap; ++j) {
-        attr = static_cast<DOM::Attr>(attrs.item(j));
-        attrName = attr.name().upper();
-
-        if( (attrName == "SRC" && (nodeName == "IMG" || nodeName == "SCRIPT")) ||
-            (attrName == "HREF" && nodeName == "LINK")) {
-/*          (attrName == "BACKGROUND" && (nodeName == "BODY" ||
-                                                       nodeName == "TABLE" ||
-                                                       nodeName == "TH" ||
-                                                       nodeName == "TD"))) */
-          elem.setAttribute(attr.name(), handleLink(attr.value().string()));
-        }
-      }
-    }
-  // now it's probably a text node
-  } else if(node_.parentNode().nodeName().upper() == "STYLE") {
-    node_.setNodeValue(analyzeInternalCSS(node_.nodeValue().string()));
-  }
-
-  DOM::Node child = node_.firstChild();
-  while(!child.isNull()) {
-    parseNode(child);
-    child = child.nextSibling();
-  }
+// how ugly is this?
+const xmlChar* HTMLExporter::handleLink(const xmlChar* link_) {
+  return reinterpret_cast<xmlChar*>(qstrdup(handleLink(QString::fromUtf8(reinterpret_cast<const char*>(link_))).utf8()));
 }
 
 QString HTMLExporter::handleLink(const QString& link_) {
@@ -492,32 +542,48 @@ QString HTMLExporter::handleLink(const QString& link_) {
   }
   // assume that if the link_ is not relative, then we don't need to copy it
   if(!KURL::isRelativeURL(link_)) {
-    m_links.insert(link_, link_);
     return link_;
   }
 
   if(m_xsltFilePath.isEmpty()) {
     m_xsltFilePath = locate("appdata", m_xsltFile);
     if(m_xsltFilePath.isNull()) {
-      myDebug() << "HTMLExporter::handleLink() - no xslt file for " << m_xsltFile << endl;
+      kdWarning() << "HTMLExporter::handleLink() - no xslt file for " << m_xsltFile << endl;
     }
   }
 
   KURL u;
   u.setPath(m_xsltFilePath);
   u = KURL(u, link_);
-  m_files.append(u);
-  m_links.insert(link_, fileDirName() + u.fileName());
-  return fileDirName() + u.fileName();
+
+  // one of the "quirks" of the html export is that img src urls are set to point to
+  // the tmpDir() when exporting entry files from a collection, but those images
+  // don't actually exist, and they get copied in writeImages() instead.
+  // so we only need to keep track of the url if it exists and is local
+  if(!u.isLocalFile() || KIO::NetAccess::exists(u, false, 0)) {
+    m_files.append(u);
+  }
+
+  // if we're exporting entry files, we want pics/ to
+  // go in pics/
+  QString midDir;
+  if(link_.startsWith(m_dataDir + QString::fromLatin1("pics/"))) {
+    midDir += QString::fromLatin1("pics/");
+  }
+  m_links.insert(link_, fileDirName() + midDir + u.fileName());
+  return m_links[link_];
+}
+
+const xmlChar* HTMLExporter::analyzeInternalCSS(const xmlChar* str_) {
+  return reinterpret_cast<xmlChar*>(qstrdup(analyzeInternalCSS(QString::fromUtf8(reinterpret_cast<const char*>(str_))).utf8()));
 }
 
 QString HTMLExporter::analyzeInternalCSS(const QString& str_) {
   QString str = str_;
   int start = 0;
   int end = 0;
-  const int length = str.length();
   const QString url = QString::fromLatin1("url(");
-  for(int pos = pos = str.find(url); pos < length && pos >= 0; pos = str.find(url, pos+1)) {
+  for(int pos = str.find(url); pos >= 0; pos = str.find(url, pos+1)) {
     pos += 4; // url(
     if(str[pos] ==  '"' || str[pos] == '\'') {
       ++pos;
@@ -535,38 +601,55 @@ QString HTMLExporter::analyzeInternalCSS(const QString& str_) {
   return str;
 }
 
+void HTMLExporter::createDir() {
+  if(!m_checkCreateDir) {
+    return;
+  }
+  KURL dir = fileDir();
+  if(dir.isEmpty()) {
+    myDebug() << "HTMLExporter::createDir() - called on empty URL!" << endl;
+    return;
+  }
+  if(KIO::NetAccess::exists(dir, false, 0)) {
+    m_checkCreateDir = false;
+  } else {
+    m_checkCreateDir = !KIO::NetAccess::mkdir(dir, m_widget);
+  }
+}
+
 bool HTMLExporter::copyFiles() {
-  bool checkTarget = true;
-  for(KURL::List::ConstIterator it = m_files.begin(); it != m_files.end(); ++it) {
+  if(m_files.isEmpty()) {
+    return true;
+  }
+  const uint start = 20;
+  const uint maxProgress = m_exportEntryFiles ? 40 : 80;
+  const uint stepSize = QMAX(1, m_files.count()/maxProgress);
+  uint j = 0;
+
+  createDir();
+  KURL target;
+  for(KURL::List::ConstIterator it = m_files.begin(); it != m_files.end() && !m_cancelled; ++it, ++j) {
     if(m_copiedFiles.has((*it).url())) {
       continue;
     }
-    m_copiedFiles.add((*it).url());
 
-    KURL target = fileDir();
-    if(checkTarget && !KIO::NetAccess::exists(target, false, 0)) {
-#if KDE_IS_VERSION(3,1,90)
-      KIO::NetAccess::mkdir(target, m_widget);
-#else
-      KIO::NetAccess::mkdir(target);
-#endif
-      checkTarget = false;
+    if(target.isEmpty()) {
+      target = fileDir();
     }
-
-    target.addPath((*it).fileName().section('/', -1));
-    // KIO::NetAccess::copy() doesn't force overwrite
-    // TODO: should the file be deleted?
-#if KDE_IS_VERSION(3,1,90)
-    KIO::NetAccess::del(target, m_widget);
-    bool success = KIO::NetAccess::copy(*it, target, m_widget);
-#else
-    KIO::NetAccess::del(target);
-    bool success = KIO::NetAccess::copy(*it, target);
-#endif
-    if(!success) {
+    target.setFileName((*it).fileName());
+    bool success = KIO::NetAccess::file_copy(*it, target, -1, true /* overwrite */, false /* resume */, m_widget);
+    if(success) {
+      m_copiedFiles.add((*it).url());
+    } else {
       kdWarning() << "HTMLExporter::copyFiles() - can't copy " << target << endl;
       kdWarning() << KIO::NetAccess::lastErrorString() << endl;
     }
+    if(j%stepSize == 0) {
+      if(options() & ExportProgress) {
+        ProgressManager::self()->setProgress(this, QMIN(start+j/stepSize, 99));
+      }
+      kapp->processEvents();
+    }
   }
   return true;
 }
@@ -577,6 +660,10 @@ bool HTMLExporter::writeEntryFiles() {
     return false;
   }
 
+  const uint start = 60;
+  const uint stepSize = QMAX(1, entries().count()/40);
+  uint j = 0;
+
   // now worry about actually exporting entry files
   // I can't reliable encode a string as a URI, so I'm punting, and I'll just replace everything but
   // a-zA-Z0-9 with an underscore. This MUST match the filename template in tellico2html.xsl
@@ -588,37 +675,130 @@ bool HTMLExporter::writeEntryFiles() {
 
   GUI::CursorSaver cs(Qt::waitCursor);
 
-  int count = 0;
-  const int processCount = 100; // process after every 100 events
-
   HTMLExporter exporter(collection());
-  exporter.setOptions(options() | Export::ExportForce);
+  long opt = options() | Export::ExportForce;
+  opt &= ~ExportProgress;
+  exporter.setOptions(opt);
   exporter.setXSLTFile(m_entryXSLTFile);
   exporter.setCollectionURL(url());
+  bool parseDOM = true;
 
+  const QString title = QString::fromLatin1("title");
+  const QString html = QString::fromLatin1(".html");
+  bool multipleTitles = collection()->fieldByName(title)->flags() & Data::Field::AllowMultiple;
   Data::EntryVec entries = this->entries(); // not const since the pointer has to be copied
-  for(Data::EntryVecIt entryIt = entries.begin(); entryIt != entries.end(); ++entryIt) {
-    QString file = entryIt->field(QString::fromLatin1("title"), formatted);
+  for(Data::EntryVecIt entryIt = entries.begin(); entryIt != entries.end() && !m_cancelled; ++entryIt, ++j) {
+    QString file = entryIt->field(title, formatted);
 
     // but only use the first title if it has multiple
-    if(collection()->fieldByName(QString::fromLatin1("title"))->flags() & Data::Field::AllowMultiple) {
+    if(multipleTitles) {
       file = file.section(';', 0, 0);
     }
-    file.replace(badChars, QString::fromLatin1("_"));
-    file += QChar('-') + QString::number(entryIt->id()) + QString::fromLatin1(".html");
+    file.replace(badChars, QChar('_'));
+    file += QChar('-') + QString::number(entryIt->id()) + html;
     outputFile.setFileName(file);
 
-    Data::EntryVec vec;
-    vec.append(entryIt);
-    exporter.setEntries(vec);
+    exporter.setEntries(Data::EntryVec(entryIt));
     exporter.setURL(outputFile);
     exporter.exec();
 
-    if(++count == processCount) {
-      kapp->processEvents();
-      count = 0;
+    // no longer need to parse DOM
+    if(parseDOM) {
+      parseDOM = false;
+      exporter.setParseDOM(false);
+      // this is rather stupid, but I'm too lazy to figure out the better way
+      // since we parsed the DOM for the first entry file to grab any
+      // images used in the template, need to resave it so the image links
+      // get written correctly
+      exporter.exec();
     }
+
+    if(j%stepSize == 0) {
+      if(options() & ExportProgress) {
+        ProgressManager::self()->setProgress(this, QMIN(start+j/stepSize, 99));
+      }
+      kapp->processEvents();
+    }
+  }
+  // the images in "pics/" are special data images, copy them always
+  QStringList dataImages;
+  dataImages << QString::fromLatin1("checkmark.png");
+  for(uint i = 1; i <= 10; ++i) {
+    dataImages << QString::fromLatin1("stars%1.png").arg(i);
+  }
+  KURL dataDir;
+  dataDir.setPath(KGlobal::dirs()->findResourceDir("appdata", QString::fromLatin1("pics/tellico.png")) + "pics/");
+  KURL target = fileDir();
+  target.addPath(QString::fromLatin1("pics/"));
+  KIO::NetAccess::mkdir(target, m_widget);
+  for(QStringList::ConstIterator it = dataImages.begin(); it != dataImages.end(); ++it) {
+    dataDir.setFileName(*it);
+    target.setFileName(*it);
+    KIO::NetAccess::copy(dataDir, target, m_widget);
   }
 
   return true;
 }
+
+void HTMLExporter::slotCancel() {
+  m_cancelled = true;
+}
+
+void HTMLExporter::parseDOM(xmlNode* node_) {
+  if(node_ == 0) {
+    myDebug() << "HTMLExporter::parseDOM() - no node" << endl;
+    return;
+  }
+
+  bool parseChildren = true;
+
+  if(node_->type == XML_ELEMENT_NODE) {
+    const QCString nodeName = QCString(reinterpret_cast<const char*>(node_->name)).upper();
+    xmlElement* elem = reinterpret_cast<xmlElement*>(node_);
+    // to speed up things, check now for nodename
+    if(nodeName == "IMG" || nodeName == "SCRIPT" || nodeName == "LINK") {
+      for(xmlAttribute* attr = elem->attributes; attr; attr = attr->nexth) {
+        QCString attrName = QCString(reinterpret_cast<const char*>(attr->name)).upper();
+
+        if( (attrName == "SRC" && (nodeName == "IMG" || nodeName == "SCRIPT")) ||
+            (attrName == "HREF" && nodeName == "LINK")) {
+/*          (attrName == "BACKGROUND" && (nodeName == "BODY" ||
+                                                       nodeName == "TABLE" ||
+                                                       nodeName == "TH" ||
+                                                       nodeName == "TD"))) */
+          xmlChar* value = xmlGetProp(node_, attr->name);
+          if(value) {
+            xmlSetProp(node_, attr->name, handleLink(value));
+            xmlFree(value);
+          }
+          // each node only has one significant attribute, so break now
+          break;
+        }
+      }
+    } else if(nodeName == "STYLE") {
+      // if the first child is a CDATA, use it, otherwise replace complete node
+      xmlNode* nodeToReplace = node_;
+      xmlNode* child = node_->children;
+      if(child && child->type == XML_CDATA_SECTION_NODE) {
+        nodeToReplace = child;
+      }
+      xmlChar* value = xmlNodeGetContent(nodeToReplace);
+      if(value) {
+        xmlNodeSetContent(nodeToReplace, analyzeInternalCSS(value));
+        xmlFree(value);
+      }
+      // no longer need to parse child text nodes
+      parseChildren = false;
+    }
+  }
+
+  if(parseChildren) {
+    xmlNode* child = node_->children;
+    while(child) {
+      parseDOM(child);
+      child = child->next;
+    }
+  }
+}
+
+#include "htmlexporter.moc"
