@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -28,6 +28,8 @@ namespace Tellico {
     extern const QString dtdTellico;
 
     extern const QString nsBookcase;
+    extern const QString nsDublinCore;
+    extern const QString nsZing;
 
     bool validXMLElementName(const QString& name);
     QString elementName(const QString& name);
