@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -26,7 +26,11 @@ namespace Tellico {
       MODS,
       Alexandria,
       FreeDB,
-      RIS
+      RIS,
+      GCfilms,
+      FileListing,
+      GRS1,
+      AMC
     };
 
     enum Action {
@@ -45,6 +49,7 @@ namespace Tellico {
   namespace Export {
     enum Format {
       TellicoXML,
+      TellicoZip,
       Bibtex,
       Bibtexml,
       HTML,
@@ -53,7 +58,8 @@ namespace Tellico {
       Text,
       PilotDB,
       Alexandria,
-      ONIX
+      ONIX,
+      GCfilms
     };
 
     enum Target {
